﻿using System;

namespace AB3.DataTypes
{
	/// <summary>Exception that is because of errors in FDL
	/// </summary>
	public class FDLException: Exception
	{
		/// <summary>Line number that caused the error
		/// </summary>
		public int LineNumber { get; private set; }

		/// <summary>The line that caused the error
		/// </summary>
		public string LineString { get; private set; }

		/// <summary>Creates new instance of a flow definition exception
		/// </summary>
		/// <param name="message">The description of the error</param>
		/// <param name="lineNumber">Where was the problem found</param>
		/// <param name="lineString">The text that caused the problem</param>
		public FDLException(string message, int lineNumber, string lineString)
			: base(message)
		{
			this.LineNumber = lineNumber;
			this.LineString = lineString;
		}

		/// <summary>Creates new intstance of a flow definition exception with inner exception
		/// </summary>
		/// <param name="message">The description of the error</param>
		/// <param name="lineNumber">Where was the problem found</param>
		/// <param name="lineString">The text that caused the problem</param>
		/// <param name="innerException">The inner exception</param>
		public FDLException(string message, int lineNumber, string lineString, Exception innerException)
			: base(message, innerException)
		{
			this.LineNumber = lineNumber;
			this.LineString = lineString;
		}

		/// <summary>Returns suitable description of the exception
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			string tReturn = string.Format("FlowDefinitionException: Message '{0}', line number {1}, line string '{2}'",
				base.Message, this.LineNumber, this.LineString);

			if (base.InnerException != null)
				tReturn += string.Format(", Inner Exception: '{0}'", base.InnerException.Message);

			return tReturn;
		}
	}
}
