﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AB3.Patterns
{
	/// <summary>Type safe implementation of converting IEnumerable to a stream 
	/// </summary>
	/// <typeparam name="T">Type of the content</typeparam>
	public class ConvertToStream<T>
	{
		private readonly T _endOfStreamValue;

		/// <summary>Typesafe implmenetation of converting IEnumerable to a stream
		/// </summary>
		public ConvertToStream() : this(default(T)) { }

		/// <summary>Typesafe implmenetation of converting IEnumerable to a stream
		/// </summary>
		/// <param name="endOfStreamValue">Value to mark the end of the stream</param>
		public ConvertToStream(T endOfStreamValue)
		{
			_endOfStreamValue = endOfStreamValue;
		}

		/// <summary>Pin to start the converting of IEnumerable values to a stream
		/// </summary>
		/// <param name="values">Content to convert to a stream</param>
		public void In_Input(IEnumerable<T> values)
		{
			foreach (var v in values)
				Out_NextValue(v);
			Out_NextValue(_endOfStreamValue);
			Out_EndOfStream();
		}

		/// <summary>Output pin for the next value of the stream
		/// </summary>
		public event Action<T> Out_NextValue;

		/// <summary>Output pin to send the end of stream signal
		/// </summary>
		public event Action Out_EndOfStream = delegate { };
	}
}
