﻿using System;
using System.Threading;

namespace AB3.Patterns
{
	#region Type safe implementation <T>
	/// <summary>Synchronizes the signal to the current context
	/// </summary>
	/// <typeparam name="T">Type of the signal</typeparam>
	public class Synchronizer<T>
	{
		private readonly SynchronizationContext synchronizationContext;

		/// <summary>Synchronizes the signal to the current context
		/// </summary>
		public Synchronizer()
		{
			synchronizationContext = SynchronizationContext.Current ?? new SynchronizationContext();
		}

		/// <summary>Input pin of the signal
		/// </summary>
		/// <param name="t">content of the signal</param>
		public void Process(T t)
		{
			synchronizationContext.Send(state => Result(t), null);
		}

		/// <summary>Output pin of the signal
		/// </summary>
		public event Action<T> Result;
	}
	#endregion

	#region Implementation for Trigger (without content)
	/// <summary>Implementation for Trigger (without content)
	/// </summary>
	public class Synchronizer
	{
		private readonly SynchronizationContext synchronizationContext;

		/// <summary>Implementation for Trigger (without content)
		/// </summary>
		public Synchronizer()
		{
			synchronizationContext = SynchronizationContext.Current ?? new SynchronizationContext();
		}

		/// <summary>Input pin of the trigger
		/// </summary>
		public void Process()
		{
			synchronizationContext.Send(state => Result(), null);
		}

		/// <summary>Output pin of the trigger
		/// </summary>
		public event Action Result;
	}
	#endregion
}
