﻿using System;
using System.IO;
using System.Reflection;
using AB3.DataTypes;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Finds assemblies in bin folder of the app
	/// </summary>
	public class AssembliesFinder
	{
		/// <summary>Starts the finding of assemblies in bin folder of the app
		/// </summary>
		/// <param name="app">The app definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding assemblies.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			Assembly tEntryAssembly = Assembly.GetEntryAssembly();
			if (tEntryAssembly != null)
			{
				string tPath = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);

				#region Find files of path and own assembly
				DirectoryInfo tDirectoryInfo = new DirectoryInfo(tPath);
				string tOwnFile = Path.GetFileName(Assembly.GetExecutingAssembly().Location);
				#endregion

				#region Generate .dll file list (except own assembly)
				foreach (FileInfo tFile in tDirectoryInfo.GetFiles("*.dll"))
				{
					// Ignore own assembly
					if (tFile.Name != tOwnFile)
						app.AssembliesToLoad.Add(tFile);
				}
				#endregion

				#region Logging & Profiling
				if (app.AssembliesToLoad.Count == 0)
				{
					app.ErrorLogger.LogWarning("No possible files found in bin folder!");
				}
				else
				{
					app.ProgressLogger.LogInfo(string.Format("Assemblies found in '{0}':", tPath));
					foreach (FileInfo tFile in app.AssembliesToLoad)
						app.ProgressLogger.LogInfo("Possible file: '{0}'", tFile.Name);
				}
				TimeSpan tDuration = DateTime.Now - tStart;
				app.BasicLogger.LogInfo("Finished finding assemblies in {0} ms.",
					tDuration.Milliseconds.ToString());
				#endregion
			}
			else
				app.BasicLogger.LogInfo("No entry assembly found.");

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
