﻿using System;
using System.Collections;
using System.Collections.Generic;
using AB3.DataTypes;
using AB3.Helpers;

namespace AB3.Steps
{
	/// <summary>Finds attribute definitions in FDL
	/// </summary>
	public class FDLAttributeDefinitionsParser
	{
		/// <summary>Starts finding of attribute definitions
		/// </summary>
		/// <param name="app">The app definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding attribute definitions.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			foreach (FDLFlowDefinition tFlowDefinition in app.FlowDefinitions.Values)
			{
				app.ProgressLogger.LogInfo("Start finding attribute definitions of flow '{0}'", tFlowDefinition.Name);

				#region Attribure of remarks
				foreach (FDLRemarkDefinition tRemarkDefinition in tFlowDefinition.Remarks.Collection.Values)
				{
					if (tRemarkDefinition.Range.StartLine != tRemarkDefinition.Range.EndLine)
					{
						for (int i = tRemarkDefinition.Range.StartLine; i <= tRemarkDefinition.Range.EndLine; i++)
						{
							foreach (FDLAttribute tAttribute in LineConverter.ToAttributes(app, i))
							{
								if (tRemarkDefinition.Attributes.ContainsKey(tAttribute.Name))
								{
									FDLException tEx = new FDLException("Attribute definition with duplicate name found!",
										i + 1, app.FlowText[i]);
									app.ErrorLogger.LogError(tEx.ToString());
								}
								else
								{
									tRemarkDefinition.Attributes.Add(tAttribute.Name, tAttribute);
									app.ProgressLogger.LogInfo("Attribute definition with name '{0}' found in line {1}",
										tAttribute.Name, (i + 1).ToString());
								}
							}
						}
					}
				}
				#endregion

				#region Attributes of interfaces
				foreach (FDLInterfaceDefinition tInterfaceDefinition in tFlowDefinition.Interfaces.Collection.Values)
				{
					if (tInterfaceDefinition.Range.StartLine != tInterfaceDefinition.Range.EndLine)
					{
						for (int i = tInterfaceDefinition.Range.StartLine; i <= tInterfaceDefinition.Range.EndLine; i++)
						{
							foreach (FDLAttribute tAttribute in LineConverter.ToAttributes(app, i))
							{
								if (tInterfaceDefinition.Attributes.ContainsKey(tAttribute.Name))
								{
									FDLException tEx = new FDLException("Attribute definition with duplicate name found!",
										i + 1, app.FlowText[i]);
									app.ErrorLogger.LogError(tEx.ToString());
								}
								else
								{
									tInterfaceDefinition.Attributes.Add(tAttribute.Name, tAttribute);
									app.ProgressLogger.LogInfo("Attribute definition with name '{0}' found in line {1}",
										tAttribute.Name, (i + 1).ToString());
								}
							}
						}
					}
				}
				#endregion

				#region Attributes of steps
				foreach (FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
				{
					if (tStepDefinition.Range.StartLine != tStepDefinition.Range.EndLine)
					{
						for (int i = tStepDefinition.Range.StartLine; i <= tStepDefinition.Range.EndLine; i++)
						{
							foreach (FDLAttribute tAttribute in LineConverter.ToAttributes(app, i))
							{
								if (tStepDefinition.Attributes.ContainsKey(tAttribute.Name))
								{
									FDLException tEx = new FDLException("Attribute definition with duplicate name found!",
										i + 1, app.FlowText[i]);
									app.ErrorLogger.LogError(tEx.ToString());
								}
								else
								{
									tStepDefinition.Attributes.Add(tAttribute.Name, tAttribute);
									app.ProgressLogger.LogInfo("Attribute definition with name '{0}' found in line {1}",
										tAttribute.Name, (i + 1).ToString());
								}
							}
						}
					}
				}
				#endregion

				#region Attributes of wires
				foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
				{
					if (tWireDefinition.Range.StartLine != tWireDefinition.Range.EndLine)
					{
						for (int i = tWireDefinition.Range.StartLine; i <= tWireDefinition.Range.EndLine; i++)
						{
							foreach (FDLAttribute tAttribute in LineConverter.ToAttributes(app, i))
							{
								if (tWireDefinition.Attributes.ContainsKey(tAttribute.Name))
								{
									FDLException tEx = new FDLException("Attribute definition with duplicate name found!",
										i + 1, app.FlowText[i]);
									app.ErrorLogger.LogError(tEx.ToString());
								}
								else
								{
									tWireDefinition.Attributes.Add(tAttribute.Name, tAttribute);
									app.ProgressLogger.LogInfo("Attribute definition with name '{0}' found in line {1}",
										tAttribute.Name, (i + 1).ToString());
								}
							}
						}
					}
				}
				#endregion
			}


			#region Logging
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished attribute definitions in {1} ms.",
				app.FlowDefinitions.Count.ToString(), tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
