﻿using System;
using System.Collections.Generic;
using System.IO;
using AB3.DataTypes;

namespace AB3.Steps
{
	/// <summary>Loads FDL file from filesystem
	/// </summary>
	public class FDLFileLoader
	{
		/// <summary>Starts the loading
		/// </summary>
		/// <param name="app">The app definition</param>
		/// <param name="pathFileName">The path and filename of the file to load</param>
		public void In_Input(AppDefinition app, string pathFileName)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start reading file '{0}'", pathFileName);
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion			

			if (!File.Exists(pathFileName))
			{
				FDLException tEx = new FDLException("The given filename doesn't exist!", 0, string.Empty);
				app.ParseExceptions.Add(tEx);
				app.ErrorLogger.LogError(tEx.Message);
				return;
			}

			string[] tLines = File.ReadAllLines(pathFileName);
			app.FlowText = new List<string>(tLines);

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished reading file in {0} ms.", 
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
