﻿using System;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Finds remarks in the given FDL
	/// </summary>
	public class FDLRemarkDefinitionsParser
	{
		/// <summary>Starts finding of remarks
		/// </summary>
		/// <param name="app">The app definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding remark definitions.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			foreach (FDLFlowDefinition tFlowDefinition in app.FlowDefinitions.Values)
			{
				app.ProgressLogger.LogInfo("Start finding remarks of flow '{0}'", tFlowDefinition.Name);

				for (int i = tFlowDefinition.Range.StartLine; i <= tFlowDefinition.Range.EndLine; i++)
				{
					if (app.FlowText[i].StartsWith("Remark:"))
					{
						FDLRemarkDefinition tRemarkDefinition = new FDLRemarkDefinition();
						tRemarkDefinition.Name = LineConverter.ToRemarkDefinitionName(app.FlowText[i]);
						if (tFlowDefinition.Remarks.Collection.ContainsKey(tRemarkDefinition.Name))
						{
							FDLException tEx = new FDLException("Remark definition with duplicate name found!",
							i + 1, app.FlowText[i]);
							app.ParseExceptions.Add(tEx);
							app.ErrorLogger.LogError(tEx.ToString());
						}
						else
						{
							tFlowDefinition.Remarks.Collection.Add(tRemarkDefinition.Name, tRemarkDefinition);
							app.ProgressLogger.LogInfo("Remark definition with name '{0}' found in line {1}",
								tRemarkDefinition.Name, (i + 1).ToString());
						}

						if (app.FlowText[i].EndsWith("{"))
							tRemarkDefinition.Range.StartLine = i;
						else
							tRemarkDefinition.Range.StartLine = StringEx.FindContentStart(app.FlowText, i + 1);

						if (tRemarkDefinition.Range.StartLine != -1)
						{
							tRemarkDefinition.Range.EndLine = StringEx.FindContentEnd(app.FlowText, tRemarkDefinition.Range.StartLine);

							if (tRemarkDefinition.Range.EndLine != -1)
							{
								i = tRemarkDefinition.Range.EndLine;
							}
							else
							{
								FDLException tEx = new FDLException("End of remark definition is missing",
								i + 1, app.FlowText[i]);
								app.ParseExceptions.Add(tEx);
								app.ErrorLogger.LogError(tEx.ToString());
							}
						}
						else // No content defined => OK
						{
							tRemarkDefinition.Range.StartLine = i;
							tRemarkDefinition.Range.EndLine = i;
						}						
					}
					
				}

				app.ProgressLogger.LogInfo("{0} remarks found.", tFlowDefinition.Remarks.Collection.Count.ToString());
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished finding remarks in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
