﻿using System;
using AB3.DataTypes;
using AB3.Interfaces;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Configures all steps which implement IConfigure
	/// </summary>
	public class StepInstancesConfigurator
	{
		/// <summary>Starts the configuring
		/// </summary>
		/// <param name="app">The app definition to work with</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start configuring steps.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			IConfigure tConfigureInstance;
			foreach (StepInstance tStepInstance in app.StepInstances.Values)
			{
				if (tStepInstance.StepObject != null)
				{
					tConfigureInstance = tStepInstance.StepObject as IConfigure;
					if (tConfigureInstance != null)
					{
						app.ProgressLogger.LogInfo("Configure step '{0}'", tStepInstance.StepType.ToString());
						tConfigureInstance.Configure(app.ConfigureArg);
					}
				}
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished configuring steps in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
