﻿using System;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AB3.AppBuilderTests
{
	[TestClass]
	public class Transform0Tests
	{
		private string _DSLFolder;
		private string _FlowFile;
		private string _LogFile;

		public Transform0Tests()
		{
			_DSLFolder = @"F:\Visual Studio Projects\AB3.AppBuilder\AB3.Iteration.10.04\AB3.DSLs";
			_FlowFile = Path.Combine(_DSLFolder, "FD.Transform0.ab3");
			_LogFile = @"F:\Visual Studio Projects\AB3.AppBuilder\AB3.AppBuilder.log";
		}

		[TestMethod]
		public void Load0FromABFileTestMethod1()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 1);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tRangeStart = tAppBuilder._AppDefinition.FlowDefinitions["MainFlow"].Range.StartLine;
			int tRangeEnd = tAppBuilder._AppDefinition.FlowDefinitions["MainFlow"].Range.EndLine;

			Assert.IsTrue(10 < (tRangeStart + tRangeEnd));
		}

		[TestMethod]
		public void Load0FromABFileTestMethod2()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tAttributeCount = tAppBuilder._AppDefinition.FlowDefinitions["MainFlow"].Remarks.Collection["Remark1"].Attributes.Count;

			Assert.AreEqual(4, tAttributeCount);
		}

		[TestMethod]
		public void Load0FromABFileTestMethod3()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			Assert.AreEqual(3, tAppBuilder._AppDefinition.FlowDefinitions["MainFlow"].Steps.Collection.Count);
		}

		[TestMethod]
		public void AddPreparedStepInstance0TestMethod1()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter", this);

			Assert.IsTrue(tAppBuilder._AppDefinition.PreparedInstances.ContainsKey("MainFlow.TestAdapter"));
		}

		[TestMethod]
		public void AddPreparedStepInstance0TestMethod2()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", this);

			Assert.IsTrue(tAppBuilder._AppDefinition.PreparedInstances.ContainsKey("MainFlow.Transform0Tests"));
		}

		[TestMethod]
		public void BuildApp0TestMethod1()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter", this);
			tAppBuilder.BuildApp(null, null);

			Assert.AreNotEqual(0, tAppBuilder._AppDefinition.StepInstances.Count);
		}

		[TestMethod]
		public void BuildApp0TestMethod2()
		{
			AppBuilder tAppBuilder = new AppBuilder();
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter", this);
			tAppBuilder.BuildApp(null, null);

			Assert.AreNotEqual(0, tAppBuilder._AppDefinition.StepInstances.Count);
		}

		[TestMethod]
		public void TestFlowDefinition0_Method2()
		{
			AppBuilder tAppBuilder = new AppBuilder();
			tAppBuilder.LoadFlowsFromFile(@"F:\Visual Studio Projects\AB3.AppBuilder\AB3.Iteration.19.02\AB3.DSLs\FD.Transform0.ab3");

			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter", this);
			tAppBuilder.BuildApp(null, null);

			this.Out_Output("This is my test string!");
			Assert.AreEqual("!GNIRTS TSET YM SI SIHT", _Result);
		}

		public event Action<string> Out_Output;

		private string _Result = null;
		private Exception _Exception = null;

		public void In_Input(string result)
		{
			_Result = result;
		}

		public void ExceptionPin(Exception exception)
		{
			_Exception = exception;
		}
	}
}
