﻿using System;
using System.Diagnostics;
using System.IO;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AB3.AppBuilderTests
{
	[TestClass]
	public class Transform1Tests
	{
		private string _DSLFolder;
		private string _FlowFile;
		private string _LogFile;

		public Transform1Tests()
		{
			_DSLFolder = @"F:\Visual Studio Projects\AB3.AppBuilder\AB3.Iteration.10.04\AB3.DSLs";
			_FlowFile = Path.Combine(_DSLFolder, "FD.Transform1.ab3");
			_LogFile = @"F:\Visual Studio Projects\AB3.AppBuilder\AB3.AppBuilder.log";
		}

		[TestMethod]
		public void Load1FromABFileTestMethod1()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 1);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tFlowDefinitions = tAppBuilder._AppDefinition.FlowDefinitions.Count;
			int tStartRange = tAppBuilder._AppDefinition.FlowDefinitions["Transform"].Range.StartLine;
			int tEndRange = tAppBuilder._AppDefinition.FlowDefinitions["Transform"].Range.EndLine;

			Assert.IsTrue((tEndRange - tStartRange) > 0);
		}

		[TestMethod]
		public void Load1FromABFileTestMethod2()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tAttributeCount = tAppBuilder._AppDefinition.FlowDefinitions["MainFlow"].Remarks.Collection["Remark1"].Attributes.Count;

			Assert.AreEqual(1, tAttributeCount);			
		}

		[TestMethod]
		public void Load1FromABFileTestMethod4()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tStepCount = tAppBuilder._AppDefinition.FlowDefinitions["Transform"].Steps.Collection.Count;

			Assert.AreEqual(2, tStepCount);
		}

		[TestMethod]
		public void Load1FromABFileTestMethod5()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);

			int tInterfacesCount = tAppBuilder._AppDefinition.FlowDefinitions["Transform"].Interfaces.Collection.Count;

			Assert.AreEqual(3, tInterfacesCount);
		}

		[TestMethod]
		public void BuildApp1TestMethod1()
		{
			AppBuilder tAppBuilder = new AppBuilder(_LogFile, 2);
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter1", this);
			tAppBuilder.BuildApp(null, null);

			Assert.AreNotEqual(0, tAppBuilder._AppDefinition.StepInstances.Count);
		}

		[TestMethod]
		public void BuildApp1TestMethod2()
		{
			AppBuilder tAppBuilder = new AppBuilder();
			tAppBuilder.LoadFlowsFromFile(_FlowFile);
			tAppBuilder.AddPreparedStepInstance("MainFlow", "TestAdapter1", this);
			tAppBuilder.BuildApp(null, null);

			Assert.AreNotEqual(0, tAppBuilder._AppDefinition.StepInstances.Count);
		}
	}
}
