﻿
using System;
using System.Windows;
using System.Windows.Controls;
using AB3.DataTypes;
using AB3.AppDesigner.DataTypes;

namespace AB3.AppDesigner.Presenters
{
	public class DocumentOutlinePresenter
	{
		private TreeView _ApplicationOutlineTreeView;
		private Document _Document;
		private object _DummyNode = null;

		public DocumentOutlinePresenter()
		{
			this._ApplicationOutlineTreeView = MainWindow.Instance.treeViewApplicationOutline;
			this._ApplicationOutlineTreeView.MouseDoubleClick += ApplicationOutlineTreeView_MouseDoubleClick;
		}

		public void In_Document(Document document)
		{
			if (this._ApplicationOutlineTreeView.CheckAccess())
			{
				this._Document = document;

				this._ApplicationOutlineTreeView.Items.Clear();

				if (this._Document.Application.FlowDefinitions.Count == 0)
					return;
				if (!this._Document.Application.FlowDefinitions.ContainsKey("MainFlow"))
					return;

				FDLFlowDefinition tMainFlow = this._Document.Application.FlowDefinitions["MainFlow"];
				FDLStepDefinition tMainStep = new FDLStepDefinition();
				tMainStep.TypeName = tMainFlow.Name;
				tMainStep.IsSubFlowStep = true;

				TreeViewItem tMainFlowItem = new TreeViewItem();
				tMainFlowItem.Header = tMainFlow.Name;
				tMainFlowItem.Tag = tMainStep;
				tMainFlowItem.FontWeight = FontWeights.Heavy;
				tMainFlowItem.Items.Add(this._DummyNode);
				this._ApplicationOutlineTreeView.Items.Add(tMainFlowItem);
				tMainFlowItem.Expanded += new RoutedEventHandler(TreeViewItem_Expanded);
				tMainFlowItem.IsExpanded = true;
			}
			else
				this._ApplicationOutlineTreeView.Dispatcher.Invoke(new Action<Document>(In_Document), document);
		}

		private void TreeViewItem_Expanded(object sender, RoutedEventArgs e)
		{
			TreeViewItem tCurrentItem = (TreeViewItem)sender;
			if ((tCurrentItem.Items.Count == 1) && (tCurrentItem.Items[0] == this._DummyNode))
			{
				tCurrentItem.Items.Clear();
				FDLStepDefinition tCurrentStep = (FDLStepDefinition)tCurrentItem.Tag;
				if (tCurrentStep.IsSubFlowStep)
				{
					if (_Document.Application.FlowDefinitions.ContainsKey(tCurrentStep.TypeName))
					{
						FDLFlowDefinition tCurrentFlow = _Document.Application.FlowDefinitions[tCurrentStep.TypeName];
						foreach (FDLStepDefinition tStepDefinition in tCurrentFlow.Steps.Collection.Values)
						{
							if (tStepDefinition.IsSubFlowStep)
							{
								TreeViewItem tNewItem = new TreeViewItem();
								string tHeaderString;
								if (tStepDefinition.CustomName != tStepDefinition.TypeName)
									tHeaderString = tStepDefinition.TypeName + " (" + tStepDefinition.CustomName + ")";
								else
									tHeaderString = tStepDefinition.TypeName;
								tNewItem.Header = tHeaderString;
								tNewItem.Tag = tStepDefinition;
								tNewItem.FontWeight = FontWeights.Normal;
								tNewItem.Items.Add(this._DummyNode);
								tCurrentItem.Items.Add(tNewItem);
								tNewItem.Expanded += new RoutedEventHandler(TreeViewItem_Expanded);
								tNewItem.IsExpanded = true;
							}
						}
					}
				}
			}
		}

		private void ApplicationOutlineTreeView_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this._ApplicationOutlineTreeView.SelectedItem != null)
			{
				TreeViewItem tSelectedTreeViewItem = this._ApplicationOutlineTreeView.SelectedItem as TreeViewItem;
				if (tSelectedTreeViewItem != null)
				{
					FDLStepDefinition tStepDef = (FDLStepDefinition)tSelectedTreeViewItem.Tag;
					if (this._Document.Application.FlowDefinitions.ContainsKey(tStepDef.TypeName))
					{
						FDLFlowDefinition tFlowDef = this._Document.Application.FlowDefinitions[tStepDef.TypeName];
						Out_FlowSelected(tFlowDef);
					}
				}
			}
		}

		public event Action<FDLFlowDefinition> Out_FlowSelected;
	}
}
