﻿using System;
using System.Collections.Generic;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.AppDesigner.DataTypes;

namespace AB3.AppDesigner.Presenters
{
	public class DocumentPresenter
	{
		private Document _Document;
		private bool _InInitialisation = false;

		public DocumentPresenter()
		{
			MainWindow.Instance.textEditor.TextChanged += textEditor_TextChanged;
		}

		private void textEditor_TextChanged(object sender, EventArgs e)
		{
			if (this._Document != null)
			{
				this._Document.Text = MainWindow.Instance.textEditor.Text;
				if (!this._InInitialisation)
					this.Out_DocumentChanged(MainWindow.Instance.textEditor.Text);
			}
		}
		public event Action<string> Out_DocumentChanged;

		public void In_Document(Document document)
		{
			this._Document = document;
			this._InInitialisation = true;
			MainWindow.Instance.textEditor.Text = string.Join(Environment.NewLine, this._Document.Lines);
			this._InInitialisation = false;
		}

		public void In_NavigateToException(FDLException exception)
		{
			if ((exception.LineNumber != 0) && (!string.IsNullOrEmpty(exception.LineString)))
			{
				MainWindow.Instance.textEditor.ScrollToLine(exception.LineNumber);
				int tLength = exception.LineString.Length;

				int tCheckLineNumber = 0;
				int tStartPosition = 0;

				while (tCheckLineNumber == 0)
				{
					tStartPosition = this._Document.Text.IndexOf(exception.LineString, tStartPosition);
					if (tStartPosition == -1)
						break;

					string tCheckString = this._Document.Text.Substring(0, tStartPosition);
					tCheckLineNumber = StringEx.CountInText(tCheckString, Environment.NewLine) + 1;
					if (tCheckLineNumber != exception.LineNumber)
					{
						tCheckLineNumber = 0;
						tStartPosition = tStartPosition + tLength;
					}
				}

				
				if (tStartPosition != -1) 
					MainWindow.Instance.textEditor.Select(tStartPosition, tLength);				
			}
			else
				MainWindow.Instance.textEditor.ScrollToHome();
		}

		public void In_NavigateToFlow(FDLFlowDefinition flowDefinition)
		{
			List<string> tSearchValues = new List<string>();
			tSearchValues.Add("FlowDefinition: " + flowDefinition.Name + " ");
			tSearchValues.Add("FlowDefinition:" + flowDefinition.Name + " ");
			tSearchValues.Add("FlowDefinition: " + flowDefinition.Name + "{");
			tSearchValues.Add("FlowDefinition: " + flowDefinition.Name + " {");
			tSearchValues.Add(flowDefinition.Name);

			int tPossiblePosition = GetLastIndex(this._Document.Text, tSearchValues);
			if (tPossiblePosition == -1)
				MainWindow.Instance.textEditor.ScrollToHome();
			else
			{
				//bool tFound = false;
				int tFlowDefinitionPosition = this._Document.Text.IndexOf(flowDefinition.Name, tPossiblePosition);

				if (tFlowDefinitionPosition != -1)
				{
					string tText = this._Document.Text.Substring(0, tPossiblePosition);
					int tLineNumer = StringEx.CountInText(tText, Environment.NewLine) + 1;
					MainWindow.Instance.textEditor.ScrollToLine(tLineNumer);
					MainWindow.Instance.textEditor.Select(tFlowDefinitionPosition, flowDefinition.Name.Length);
				}				
			}
		}

		private int GetLastIndex(string text, IEnumerable<string> values)
		{
			int tPosition;

			foreach (string value in values)
			{
				tPosition = text.IndexOf(value);
				if (tPosition != -1)
					return tPosition;
			}

			return -1;
		}
	}
}
