﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using AB3.Interfaces;
using AB3.AppDesigner.DataTypes;

namespace AB3.AppDesigner.Presenters
{
	public class StepsPresenter: IInitialize
	{
		private Document _Document;

		public StepsPresenter()
		{
			MainWindow.Instance.listViewComponents.SelectionChanged += _ComponentsListView_SelectionChanged;
			MainWindow.Instance.textBoxComponentsFilter.KeyDown += textBoxComponentsFilter_KeyDown;
		}
		
		public void In_Document(Document document)
		{
			if (MainWindow.Instance.listViewComponents.CheckAccess())
			{
				this._Document = document;
				List<string> tComponents = new List<string>();

				foreach (AB3.DataTypes.FDLFlowDefinition tFlowDefinition in document.Application.FlowDefinitions.Values)
				{
					foreach (AB3.DataTypes.FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
					{
						string tStepName;
						if (tStepDefinition.IsSubFlowStep)
							tStepName = "Flow: " + tStepDefinition.TypeName;
						else
							tStepName = "Step: " + tStepDefinition.TypeName;

						if (!tComponents.Contains(tStepName))
							tComponents.Add(tStepName);
					}
				}
				
				MainWindow.Instance.listViewComponents.ItemsSource = tComponents;
				MainWindow.Instance.listViewComponents.Items.Filter = new Predicate<object>(Contains);
			}
			else
				MainWindow.Instance.listViewComponents.Dispatcher.Invoke(new Action<Document>(In_Document), document);
		}

		private void _ComponentsListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			if (MainWindow.Instance.listViewComponents.SelectedItems.Count == 1)
			{
				string tSelectedListViewItem = (string)MainWindow.Instance.listViewComponents.SelectedItem;
				this.Out_SelectedComponent(this._Document, tSelectedListViewItem);
			}
		}

		private void textBoxComponentsFilter_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			if (e.Key == System.Windows.Input.Key.Return)
			{
				MainWindow.Instance.listViewComponents.Items.Filter = new Predicate<object>(Contains);
			}
		}

		public event Action<Document, string> Out_SelectedComponent;

		#region Private Helpers: Contains
		private bool Contains(object listBoxItem)
		{
			string tSearchString = listBoxItem.ToString().ToUpper();

			if (string.IsNullOrEmpty(MainWindow.Instance.textBoxComponentsFilter.Text))
				return true;
			else
			{
				string[] tFilters = MainWindow.Instance.textBoxComponentsFilter.Text.ToUpper().Split(' ');
				bool tContains = true;
				foreach (string tFilter in tFilters)
				{
					if (!tSearchString.Contains(tFilter))
					{
						tContains = false;
						break;
					}
				}

				return tContains;
			}
		}
		#endregion

		public void Initialize(object args)
		{
			//See: http://social.msdn.microsoft.com/Forums/en-US/wpf/thread/bfb49072-d3f3-4f2a-9f83-5a53ef8c5ee2/
			MainWindow.Instance.listViewComponents.Items.SortDescriptions.Add(
				new System.ComponentModel.SortDescription());
		}
	}
}
