﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Effects;
using AB3.AppDesigner.Converters;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.Interfaces;

namespace AB3.AppDesigner.RenderFlow
{
	public class StepsRenderer
	{
		public void In_Input(ChartDefinition chartDefinition)
		{
			if (!chartDefinition.Parent.CheckAccess())
			{
				chartDefinition.Parent.Dispatcher.Invoke(new Action<ChartDefinition>(In_Input), chartDefinition);
			}
			else
			{
				//See: http://msdn.microsoft.com/de-de/library/vstudio/ms748273.aspx
				DropShadowEffect tShadowEffect = new DropShadowEffect();
				tShadowEffect.ShadowDepth = 4;
				tShadowEffect.Color = Colors.Black;
				tShadowEffect.Opacity = 0.5;
				tShadowEffect.BlurRadius = 4;

				SolidColorBrush tGoldBrush = new SolidColorBrush(Colors.Gold);
				SolidColorBrush tLightBlueBrush = new SolidColorBrush(Colors.LightBlue);
				SolidColorBrush tBlackBrush = new SolidColorBrush(Colors.Black);
				Thickness tStepBorderThickness = new Thickness(1);
				Thickness tFlowBorderThickness = new Thickness(2);
				CornerRadius tCornerRadius = new CornerRadius(4);

				foreach (FDLStepDefinition tStep in chartDefinition.FlowDefintion.Steps.Collection.Values)
				{
					TextBlock tTextBlock = new TextBlock();
					if (tStep.TypeName != tStep.CustomName)
						tTextBlock.Text = StringEx.SplitCamelCase(string.Concat(tStep.TypeName, " => ", tStep.CustomName));
					else
						tTextBlock.Text = StringEx.SplitCamelCase(tStep.TypeName);
					tTextBlock.TextWrapping = TextWrapping.Wrap;
					tTextBlock.HorizontalAlignment = HorizontalAlignment.Center;
					tTextBlock.VerticalAlignment = VerticalAlignment.Center;
					Border tBorder = new Border();
					if (tStep.IsSubFlowStep)
					{
						tBorder.Background = tGoldBrush;
						tBorder.BorderThickness = tFlowBorderThickness;
					}
					else
					{
						tBorder.Background = tLightBlueBrush;
						tBorder.BorderThickness = tStepBorderThickness;
					}
					tBorder.Child = tTextBlock;
					tBorder.BorderBrush = tBlackBrush;
					tBorder.CornerRadius = tCornerRadius;
					tBorder.Margin = FDLAttributeConverter.ToPosition(tStep.Attributes, 10, 10);
					tBorder.HorizontalAlignment = HorizontalAlignment.Left;
					tBorder.VerticalAlignment = VerticalAlignment.Top;
					FDLAttributeConverter.SetSize(tStep.Attributes, tBorder, 100, 50);
					tBorder.Effect = tShadowEffect;
					chartDefinition.Steps.Add(tStep.CustomName, tBorder);
					chartDefinition.Parent.Children.Add(tBorder);
				}
			}

			Out_Output(chartDefinition);
		}

		public event Action<ChartDefinition> Out_Output;
	}
}
