﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;
using AB3.Interfaces;

namespace AB3.AppDesigner.RenderFlow
{
	public class WiresRenderer
	{
		public void In_Input(ChartDefinition chartDefinition)
		{
			if (!chartDefinition.Parent.CheckAccess())
			{
				chartDefinition.Parent.Dispatcher.Invoke(new Action<ChartDefinition>(In_Input), chartDefinition);
			}
			else
			{
				Border tSource;
				Border tTarget;
				foreach (FDLWireDefinition tWire in chartDefinition.FlowDefintion.Wires.Collection)
				{
					tSource = null;
					tTarget = null;
					if ((tWire.Source.Step == "In") && (chartDefinition.Interfaces.ContainsKey(tWire.Source.Pin)))
						tSource = chartDefinition.Interfaces[tWire.Source.Pin];
					else if (chartDefinition.Steps.ContainsKey(tWire.Source.Step))
						tSource = chartDefinition.Steps[tWire.Source.Step];
															
					if ((tWire.Target.Step == "Out") && (chartDefinition.Interfaces.ContainsKey(tWire.Target.Pin)))
						tTarget = chartDefinition.Interfaces[tWire.Target.Pin];
					else if (chartDefinition.Steps.ContainsKey(tWire.Target.Step))
						tTarget = chartDefinition.Steps[tWire.Target.Step];

					if ((tSource != null) && (tTarget != null))
					{
						Line tLine = new Line();
						//tLine.Stroke = System.Windows.Media.Brushes.Black;
						tLine.Stroke = System.Windows.Media.Brushes.DarkCyan;

						if (tWire.Attributes.ContainsKey("SourceEdge"))
						{
							string tSourceEdge = tWire.Attributes["SourceEdge"].Value;
							if (tSourceEdge == "Left")
							{
								tLine.X1 = tSource.Margin.Left;
								tLine.Y1 = tSource.Margin.Top + tSource.Height / 2;
							}
							else if (tSourceEdge == "Top")
							{
								tLine.X1 = tSource.Margin.Left + tSource.Width / 2;
								tLine.Y1 = tSource.Margin.Top;
							}
							else if (tSourceEdge == "Right")
							{
								tLine.X1 = tSource.Margin.Left + tSource.Width;
								tLine.Y1 = tSource.Margin.Top + tSource.Height / 2;
							}
							else if (tSourceEdge == "Bottom")
							{
								tLine.X1 = tSource.Margin.Left + tSource.Width / 2;
								tLine.Y1 = tSource.Margin.Top + tSource.Height;
							}
						}
						else
						{
							if (tSource.Margin.Left == tTarget.Margin.Left)
								tLine.X1 = tSource.Margin.Left + tSource.Width / 2;
							else if (tSource.Margin.Left < tTarget.Margin.Left)
								tLine.X1 = tSource.Margin.Left + tSource.Width;
							else
								tLine.X1 = tSource.Margin.Left;

							if (tSource.Margin.Top == tTarget.Margin.Top)
								tLine.Y1 = tSource.Margin.Top + tSource.Height / 2;
							else if (tSource.Margin.Top < tTarget.Margin.Top)
								tLine.Y1 = tSource.Margin.Top + tSource.Height;
							else
								tLine.Y1 = tSource.Margin.Top;
						}

						string tTargetEdge;
						if (tWire.Attributes.ContainsKey("TargetEdge"))
						{
							tTargetEdge = tWire.Attributes["TargetEdge"].Value;

							if (tTargetEdge == "Left")
							{
								tLine.X2 = tTarget.Margin.Left;
								tLine.Y2 = tTarget.Margin.Top + tTarget.Height / 2;
							}
							else if (tTargetEdge == "Top")
							{
								tLine.X2 = tTarget.Margin.Left + tTarget.Width / 2;
								tLine.Y2 = tTarget.Margin.Top;
							}
							else if (tTargetEdge == "Right")
							{
								tLine.X2 = tTarget.Margin.Left + tTarget.Width;
								tLine.Y2 = tTarget.Margin.Top + tTarget.Height / 2;
							}
							else if (tTargetEdge == "Bottom")
							{
								tLine.X2 = tTarget.Margin.Left + tTarget.Width / 2;
								tLine.Y2 = tTarget.Margin.Top + tTarget.Height;
							}
						}
						else
						{
							tTargetEdge = "Left";
							if (tSource.Margin.Left == tTarget.Margin.Left)
							{
								tLine.X2 = tLine.X1;
								if (tLine.Y2 > tLine.Y1)
									tTargetEdge = "Top";
								else
									tTargetEdge = "Bottom";
							}
							else if (tSource.Margin.Left < tTarget.Margin.Left)
								tLine.X2 = tTarget.Margin.Left;
							else
							{
								tLine.X2 = tTarget.Margin.Left + tTarget.Width;
								tTargetEdge = "Right";
							}

							if (tSource.Margin.Top == tTarget.Margin.Top)
							{
								tLine.Y2 = tLine.Y1;
								if (tLine.X2 < tLine.X1)
									tTargetEdge = "Right";
							}
							else if (tSource.Margin.Top < tTarget.Margin.Top)
							{
								tLine.Y2 = tTarget.Margin.Top;
								tTargetEdge = "Top";
							}
							else
							{
								tLine.Y2 = tTarget.Margin.Top + tTarget.Height;
								tTargetEdge = "Bottom";
							}
						}

						tLine.HorizontalAlignment = HorizontalAlignment.Left;
						tLine.VerticalAlignment = VerticalAlignment.Top;
						tLine.StrokeThickness = 2;
						chartDefinition.Parent.Children.Add(tLine);

						const int tIndicatorTriangleWidth = 8;
						PolyLineSegment tArrowSegment = new PolyLineSegment();
						tArrowSegment.Points.Add(new Point(tLine.X2, tLine.Y2));
						if (tTargetEdge == "Left")
						{
							tArrowSegment.Points.Add(new Point(tLine.X2 - tIndicatorTriangleWidth, tLine.Y2 - tIndicatorTriangleWidth));
							tArrowSegment.Points.Add(new Point(tLine.X2 - tIndicatorTriangleWidth, tLine.Y2 + tIndicatorTriangleWidth));
						}
						else if (tTargetEdge == "Right")
						{
							tArrowSegment.Points.Add(new Point(tLine.X2 + tIndicatorTriangleWidth, tLine.Y2 - tIndicatorTriangleWidth));
							tArrowSegment.Points.Add(new Point(tLine.X2 + tIndicatorTriangleWidth, tLine.Y2 + tIndicatorTriangleWidth));
						}
						else if (tTargetEdge == "Top")
						{
							tArrowSegment.Points.Add(new Point(tLine.X2 - tIndicatorTriangleWidth, tLine.Y2 - tIndicatorTriangleWidth));
							tArrowSegment.Points.Add(new Point(tLine.X2 + tIndicatorTriangleWidth, tLine.Y2 - tIndicatorTriangleWidth));
						}
						else if (tTargetEdge == "Bottom")
						{
							tArrowSegment.Points.Add(new Point(tLine.X2 - tIndicatorTriangleWidth, tLine.Y2 + tIndicatorTriangleWidth));
							tArrowSegment.Points.Add(new Point(tLine.X2 + tIndicatorTriangleWidth, tLine.Y2 + tIndicatorTriangleWidth));
						}
						tArrowSegment.Points.Add(new System.Windows.Point(tLine.X2, tLine.Y2));
						PathFigure tArrowPathFigure = new PathFigure();
						tArrowPathFigure.Segments.Add(tArrowSegment);
						PathGeometry tArrowPathGeometry = new PathGeometry();
						tArrowPathGeometry.Figures.Add(tArrowPathFigure);
						Path tArrowPath = new Path();
						tArrowPath.Data = tArrowPathGeometry;
						tArrowPath.Fill = System.Windows.Media.Brushes.DarkCyan;
						chartDefinition.Parent.Children.Add(tArrowPath);
					}				
				}
			}
		}
	}
}
