﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AB3.DataTypes;
using AB3.AppDesigner.DataTypes;

namespace AB3.AppDesigner.Validators
{
	public class ValidateIfStepsOfWiresExist
	{
		public void In_Document(Document document)
		{
			foreach (FDLFlowDefinition tFlowDefinition in document.Application.FlowDefinitions.Values)
			{
				foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
				{
					if (tWireDefinition.Source.Step != "In")
					{
						if (!tFlowDefinition.Steps.Collection.ContainsKey(tWireDefinition.Source.Step))
						{
							FDLException tEx = new FDLException("Step of wire source is missing", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
							document.Application.ParseExceptions.Add(tEx);
						}
						else
						{
							FDLStepDefinition tStepDefinition = tFlowDefinition.Steps.Collection[tWireDefinition.Source.Step];
							if (tStepDefinition.IsSubFlowStep)
							{
								if (document.Application.FlowDefinitions.ContainsKey(tStepDefinition.TypeName))
								{
									FDLFlowDefinition tSourceFlowDefinition = document.Application.FlowDefinitions[tStepDefinition.TypeName];

									if (tSourceFlowDefinition.Interfaces.Collection.ContainsKey(tWireDefinition.Source.Pin))
									{
										if (tSourceFlowDefinition.Interfaces.Collection[tWireDefinition.Source.Pin].Direction != Directions.Out)
										{
											FDLException tEx = new FDLException("Interface at source flow has wrong direction", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
											document.Application.ParseExceptions.Add(tEx);
										}
									}
									else
									{
										FDLException tEx = new FDLException("Source flow doesn't contain interface", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
										document.Application.ParseExceptions.Add(tEx);
									}
								}
								else
								{
									FDLException tEx = new FDLException("Flow of wire source is missing", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
									document.Application.ParseExceptions.Add(tEx);
								}
							}
						}
					}

					if (tWireDefinition.Target.Step != "Out")
					{
						if (!tFlowDefinition.Steps.Collection.ContainsKey(tWireDefinition.Target.Step))
						{
							FDLException tEx = new FDLException("Step of wire target is missing", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
							document.Application.ParseExceptions.Add(tEx);
						}
						else
						{
							FDLStepDefinition tTargetStepDefinition = tFlowDefinition.Steps.Collection[tWireDefinition.Target.Step];
							if (tTargetStepDefinition.IsSubFlowStep)
							{
								if (document.Application.FlowDefinitions.ContainsKey(tTargetStepDefinition.TypeName))
								{
									FDLFlowDefinition tTargetFlowDefinition = document.Application.FlowDefinitions[tTargetStepDefinition.TypeName];
									if (tTargetFlowDefinition.Interfaces.Collection.ContainsKey(tWireDefinition.Target.Pin))
									{
										if (tTargetFlowDefinition.Interfaces.Collection[tWireDefinition.Target.Pin].Direction != Directions.In)
										{
											FDLException tEx = new FDLException("Interface at target flow has wrong direction", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
											document.Application.ParseExceptions.Add(tEx);
										}
									}
									else
									{
										FDLException tEx = new FDLException("Target flow doesn't contain In-Pin", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
										document.Application.ParseExceptions.Add(tEx);
									}
								}
								else
								{
									FDLException tEx = new FDLException("Flow of wire target is missing", tWireDefinition.LineNumber + 1, document.Lines[tWireDefinition.LineNumber]);
									document.Application.ParseExceptions.Add(tEx);
								}
							}
						}
					}
				}
			}

			this.Out_Document(document);
		}

		public event Action<Document> Out_Document;
	}
}
