﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AB3.DataTypes
{
	/// <summary>DataType to store a connection from source to target step
	/// </summary>
	public class StepInstanceConnection
	{
		/// <summary>Endpoint of the source step
		/// </summary>
		public ConnectionEndPoint SourceEndPoint { get; private set; }

		/// <summary>Endpoint of the target step
		/// </summary>
		public ConnectionEndPoint TargetEndPoint { get; private set; }

		/// <summary>Creates new instance to store a connection from source to target step
		/// </summary>
		/// <param name="sourceEndPoint">The source endpoint of the connection</param>
		/// <param name="targetEndPoint">The target endpoint of the connection</param>
		public StepInstanceConnection(ConnectionEndPoint sourceEndPoint, ConnectionEndPoint targetEndPoint)
		{
			SourceEndPoint = sourceEndPoint;
			TargetEndPoint = targetEndPoint;
		}

		/// <summary>The full name of the connection like MainFlow.SubFlow1.StepA.PinName1->MainFlowSubFlow2.StepB.PinName2
		/// </summary>
		public string FullName
		{
			get { return string.Concat(SourceEndPoint.FullName, "->", TargetEndPoint.FullName); }
		}
	}
}
