﻿using System.Collections.Generic;

namespace AB3.Helpers
{
	/// <summary>Common String Extensions
	/// </summary>
	public static class StringEx
	{
		/// <summary>Counts char in a string
		/// </summary>
		/// <param name="text">String in which to count</param>
		/// <param name="character">Character to count</param>
		/// <remarks>http://stackoverflow.com/questions/541954/how-would-you-count-occurences-of-a-string-within-a-string-c</remarks>
		/// <returns>The Count of the character</returns>
		public static int CountInText(string text, char character)
		{
			int count = 0, n = 0;
			while ((n = text.IndexOf(character, n) + 1) != 0) count++;

			return count;
		}

		/// <summary>Counts string part in a string
		/// </summary>
		/// <param name="text">String in which to count</param>
		/// <param name="part">String part to count</param>
		/// <remarks>http://stackoverflow.com/questions/541954/how-would-you-count-occurences-of-a-string-within-a-string-c</remarks>
		/// <returns>The Count of the string part</returns>
		public static int CountInText(string text, string part)
		{
			int count = 0, n = 0;
			while ((n = text.IndexOf(part, n) + 1) != 0) count++;

			return count;
		}

		/// <summary>Finds the start of content of a FDL by searching for '{'
		/// </summary>
		/// <param name="text">List of text</param>
		/// <param name="lineNumber">Line number ot text to start the search</param>
		/// <returns>The Line number of '{' or -1 if nothing was found</returns>
		public static int FindContentStart(List<string> text, int lineNumber)
		{
			while (lineNumber < text.Count)
			{
				if (text[lineNumber].StartsWith("{"))
					return lineNumber;
				//else if (text[lineNumber].EndsWith("{"))
				//	return lineNumber;
				else if (text[lineNumber].StartsWith("Wire:"))
					return -1;
				else if (text[lineNumber].StartsWith("Step:"))
					return -1;
				else if (text[lineNumber].StartsWith("Flow:"))
					return -1;
				else if (text[lineNumber].StartsWith("In:"))
					return -1;
				else if (text[lineNumber].StartsWith("Out:"))
					return -1;
				else if (text[lineNumber].StartsWith("Remark:"))
					return -1;
				else if (text[lineNumber].StartsWith("FlowDefinition:"))
					return -1;
				else
					lineNumber++;
			}

			return -1;
		}

		/// <summary>Finds the end of content of a FDL by searching for '}'
		/// </summary>
		/// <param name="text">List of text</param>
		/// <param name="lineNumber">Line number ot text to start the search</param>
		/// <returns>The Line number of '}' or -1 if nothing was found</returns>
		public static int FindContentEnd(List<string> text, int lineNumber)
		{
			int tBracketsCount = 0;
			while (lineNumber < text.Count)
			{
				tBracketsCount += StringEx.CountInText(text[lineNumber], '{');
				tBracketsCount -= StringEx.CountInText(text[lineNumber], '}');

				if (tBracketsCount == 0)
					return lineNumber;
				else
					lineNumber++;
			}

			return -1;
		}

		/// <summary>Splits a camel case by space. For example 'SplitCamelCase' to 'Split Camel Case'
		/// 
		/// </summary>
		/// <param name="text">String to split</param>
		/// <returns>Splittet text</returns>
		public static string SplitCamelCase(string text)
		{
			List<char> tCharacters = new List<char>(text.ToCharArray());
			List<int> tPositions = new List<int>();
			const string tUpperCharacters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

			for (int i = 1; i < tCharacters.Count; i++) // Ignore first letter
			{
				if (tUpperCharacters.Contains(tCharacters[i].ToString()))
				{
					tPositions.Add(i);
				}
			}

			if (tPositions.Count == 0)
				return text;
			else
			{
				for (int i = tPositions.Count - 1; i >= 0; i--)
				{
					tCharacters.Insert(tPositions[i], ' ');
				}

				string tReturn = string.Empty;
				foreach (char tChararcter in tCharacters)
				{
					tReturn += tChararcter.ToString();
				}

				return tReturn;
			}
		}
	}
}
