﻿using System;

namespace AB3.Patterns
{
	/// <summary>Join multiple input signals to single output
	/// </summary>
	/// <typeparam name="S">Type of input1 (routed to Output as first argument)</typeparam>
	/// <typeparam name="T">Type of input2 (routed to Output as second argument)</typeparam>
	public class JoinToMultipleArguments<S, T>
	{
		private S _Input1;
		private bool _Input1Present;
		private T _Input2;
		private bool _Input2Present;

		#region Ctor
		/// <summary>Creates new Instance of signal joiner to single output with multiple arugments
		/// </summary>
		public JoinToMultipleArguments()
		{
			this._Input1Present = false;
			this._Input2Present = false;
		}
		#endregion

		/// <summary>Input1 of signal
		/// </summary>
		/// <param name="input1"></param>
		public void In_Input1(S input1)
		{
			this._Input1 = input1;
			this._Input1Present = true;
			TrySendOutput();
		}

		/// <summary>Input2 of signal
		/// </summary>
		/// <param name="input2"></param>
		public void In_Input2(T input2)
		{
			this._Input2 = input2;
			this._Input2Present = true;
			TrySendOutput();
		}

		private void TrySendOutput()
		{
			if (this._Input1Present && this._Input2Present)
			{
				this._Input1Present = false;
				this._Input2Present = false;
				this.Out_Output(this._Input1, this._Input2);
			}
		}

		/// <summary>Send the input if all values given as single signal
		/// </summary>
		public event Action<S, T> Out_Output;
	}
}
