﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

namespace AB3.Patterns
{
	/// <summary>
	/// 
	/// </summary>
	/// <typeparam name="TInput"></typeparam>
	/// <typeparam name="TOutput1"></typeparam>
	/// <typeparam name="TOutput2"></typeparam>
	public class Split<TInput, TOutput1, TOutput2>
	{
		private readonly Func<TInput, TOutput1> factory1;
		private readonly Func<TInput, TOutput2> factory2;

		/// <summary>
		/// 
		/// </summary>
		public Split()
		{
			if (typeof(TOutput1) == typeof(TInput))
			{
				factory1 = x => (TOutput1)(object)x;
			}
			if (typeof(TOutput2) == typeof(TInput))
			{
				factory2 = x => (TOutput2)(object)x;
			}

			var getter1Name = "";
			if (factory1 == null)
			{
				var getter1 = GetGettersOfType(typeof(TOutput1))
					.First()
					.GetGetMethod(true);
				factory1 = x => (TOutput1)getter1.Invoke(x, new object[] { });
				getter1Name = getter1.Name;
			}

			if (factory2 == null)
			{
				var getter2 = GetGettersOfType(typeof(TOutput2))
					.Where(property => property.GetGetMethod(true).Name != getter1Name)
					.First()
					.GetGetMethod(true);
				factory2 = x => (TOutput2)getter2.Invoke(x, new object[] { });
			}
		}

		private static IEnumerable<PropertyInfo> GetGettersOfType(Type type)
		{
			return typeof(TInput).GetProperties(BindingFlags.Public | BindingFlags.Instance)
				.Where(property => property.PropertyType == type);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="factory1"></param>
		/// <param name="factory2"></param>
		public Split(Func<TInput, TOutput1> factory1, Func<TInput, TOutput2> factory2)
		{
			this.factory1 = factory1;
			this.factory2 = factory2;
		}

		/// <summary>Input signal to split
		/// </summary>
		/// <param name="message">Content of the signal</param>
		public void In_Input(TInput message)
		{
			Out_Output1(factory1(message));
			Out_Output2(factory2(message));
		}

		/// <summary>Output event of signal 1
		/// </summary>
		public event Action<TOutput1> Out_Output1;

		/// <summary>Output event of signal 2
		/// 
		/// </summary>
		public event Action<TOutput2> Out_Output2;
	}

	/// <summary>
	/// 
	/// </summary>
	public class Split
	{
		/// <summary>Input signal to split
		/// </summary>
		public void In_Input()
		{
			Out_Output1();
			Out_Output2();
		}

		/// <summary>Output event of signal 1
		/// </summary>
		public event Action Out_Output1;

		/// <summary>Output event of signal 2
		/// </summary>
		public event Action Out_Output2;
	}
}
