﻿using System;

namespace AB3.Patterns
{
	/// <summary>Splits arguments to separated signals
	/// </summary>
	/// <typeparam name="S">Type of input1 (routed to Output1)</typeparam>
	/// <typeparam name="T">Type of input2 (routed to Output2)</typeparam>
	public class SplitArguments<S, T>
	{
		/// <summary>Splits arguments to separated signals
		/// </summary>
		/// <param name="input1">Routed to Output1</param>
		/// <param name="input2">Routed to Output2</param>
		public void In_Input(S input1, T input2)
		{
			this.Out_Output1(input1);
			this.Out_Output2(input2);
		}

		/// <summary>Output1 of input1
		/// </summary>
		public event Action<S> Out_Output1;

		/// <summary>Output2 of input2
		/// </summary>
		public event Action<T> Out_Output2;
	}

	/// <summary>Splits arguments to separated signals
	/// </summary>
	/// <typeparam name="R">Type of input1 (routed to Output1)</typeparam>
	/// <typeparam name="S">Type of input2 (routed to Output2)</typeparam>
	/// <typeparam name="T">Type of input3 (routed to Output3)</typeparam>
	public class SplitArguments<R, S, T>
	{
		/// <summary>Splits arguments to separated signals
		/// </summary>
		/// <param name="input1">Routed to Output1</param>
		/// <param name="input2">Routed to Output2</param>
		/// <param name="input3">Routed to Output3</param>
		public void In_Input(R input1, S input2, T input3)
		{
			this.Out_Output1(input1);
			this.Out_Output2(input2);
			this.Out_Output3(input3);
		}

		/// <summary>Output1 of input1
		/// </summary>
		public event Action<R> Out_Output1;

		/// <summary>Output2 of input2
		/// </summary>
		public event Action<S> Out_Output2;

		/// <summary>Output3 of input3
		/// </summary>
		public event Action<T> Out_Output3;
	}
}
