﻿using System;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Finds the interface definitions of flows defined in FDL 
	/// </summary>
	public class FDLInterfaceDefinitionsParser
	{
		/// <summary>Starts the finding of interface definitions
		/// </summary>
		/// <param name="app">The app definition to work with</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding interfaces.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			foreach (FDLFlowDefinition tFlowDefinition in app.FlowDefinitions.Values)
			{
				app.ProgressLogger.LogInfo("Start finding interfaces of flow '{0}'", tFlowDefinition.Name);

				for (int i = tFlowDefinition.Range.StartLine; i <= tFlowDefinition.Range.EndLine; i++)
				{
					if ((app.FlowText[i]).StartsWith("In:") || (app.FlowText[i].StartsWith("Out:")))
					{
						FDLInterfaceDefinition tInterfaceDefinition = LineConverter.ToInterfaceDefinition(app, i);
						if (tFlowDefinition.Interfaces.Collection.ContainsKey(tInterfaceDefinition.Name))
						{
							FDLException tEx;
							if (tInterfaceDefinition.Direction == Directions.In)
								tEx = new FDLException("Incoming interface with duplicate name found!", i + 1, app.FlowText[i]);
							else
								tEx = new FDLException("Outgoing interface with duplicate name found!", i + 1, app.FlowText[i]);
							app.ParseExceptions.Add(tEx);
							app.ProgressLogger.LogError(tEx.ToString());
						}
						else
						{
							tFlowDefinition.Interfaces.Collection.Add(tInterfaceDefinition.Name, tInterfaceDefinition);
							if (tInterfaceDefinition.Direction == Directions.In)
								app.ProgressLogger.LogInfo("Incoming interface with name '{0}' found in line {1}",
									tInterfaceDefinition.Name, (i + 1).ToString());
							else
								app.ProgressLogger.LogInfo("Outgoing interface with name '{0}' found in line {1}",
									tInterfaceDefinition.Name, (i + 1).ToString());
						}

						if (app.FlowText[i].EndsWith("{"))
							tInterfaceDefinition.Range.StartLine = i;
						else
							tInterfaceDefinition.Range.StartLine = StringEx.FindContentStart(app.FlowText, i + 1);

						if (tInterfaceDefinition.Range.StartLine != -1)
						{
							tInterfaceDefinition.Range.EndLine = StringEx.FindContentEnd(app.FlowText, tInterfaceDefinition.Range.StartLine);

							if (tInterfaceDefinition.Range.EndLine != -1)
							{
								i = tInterfaceDefinition.Range.EndLine;
							}
							else
							{
								FDLException tEx = new FDLException("End of interface definition is missing",
								i + 1, app.FlowText[i]);
								app.ParseExceptions.Add(tEx);
								app.ProgressLogger.LogError(tEx.ToString());
							}
						}
						else // No content defined => OK
						{
							tInterfaceDefinition.Range.StartLine = i;
							tInterfaceDefinition.Range.EndLine = i;
						}
					}
				}
				
				app.ProgressLogger.LogInfo("{0} interfaces found.", tFlowDefinition.Interfaces.Collection.Count.ToString());
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished parsing interfaces in {0} ms.", tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
