﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AB3.DataTypes;

namespace AB3.Steps
{
	/// <summary>Converts a text (string) to a FDL in lines
	/// </summary>
	public class FDLTextConverter
	{
		/// <summary>Starts the converting
		/// </summary>
		/// <param name="app">The app definition</param>
		/// <param name="text">The text to convert</param>
		public void In_Input(AppDefinition app, string text)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start converting text with {0} characters", text.Length.ToString());
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			try
			{
				string[] tLines = text.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
				app.FlowText = new List<string>(tLines);
			}
			catch (Exception ex)
			{
				FDLException tEx = new FDLException("Unable to convert text!", 0, string.Empty, ex);
				app.ParseExceptions.Add(tEx);
				app.ErrorLogger.LogError(tEx.ToString());
			}
			

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished converting text in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
