﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using AB3.AppDesigner.Helpers;

namespace AB3.AppDesigner.Adorners
{
	//See: http://denisvuyka.wordpress.com/2007/10/15/wpf-simple-adorner-usage-with-drag-and-resize-operations/
	public class BorderAdorner: Adorner
	{
		private Thumb _TopLeft, _TopRight, _BottomLeft, _BottomRight, _TopMiddle, _BottomMiddle, _LeftMiddle, _RightMiddle, _CenterThumb;
		private VisualCollection _VisualChildren;
		private Border _AdornedElement;
		private double _OriginalWith;
		private double _OriginalHeight;
		private double _OriginalTop;
		private double _OriginalLeft;

		public BorderAdorner(Border adornedElement): base(adornedElement)
		{
			_VisualChildren = new VisualCollection(this);
			_AdornedElement = adornedElement;
			_OriginalHeight = adornedElement.Height;
			_OriginalLeft = adornedElement.Margin.Left;
			_OriginalTop = adornedElement.Margin.Top;
			_OriginalWith = adornedElement.Width;

			// Call a helper method to initialize the Thumbs
			// with a customized cursors.
			BuildAdornerCorner(ref _TopLeft, Cursors.SizeNWSE);
			BuildAdornerCorner(ref _TopMiddle, Cursors.SizeNS);
			BuildAdornerCorner(ref _TopRight, Cursors.SizeNESW);
			BuildAdornerCorner(ref _BottomLeft, Cursors.SizeNESW);
			BuildAdornerCorner(ref _BottomMiddle, Cursors.SizeNS);
			BuildAdornerCorner(ref _BottomRight, Cursors.SizeNWSE);
			BuildAdornerCorner(ref _LeftMiddle, Cursors.SizeWE);
			BuildAdornerCorner(ref _RightMiddle, Cursors.SizeWE);
			BuildAdornerCorner(ref _CenterThumb, Cursors.Hand);

			_CenterThumb.DragDelta += new DragDeltaEventHandler(HandleCenterThumb);
			_CenterThumb.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_TopLeft.DragDelta += new DragDeltaEventHandler(HandleTopLeftThumb);
			_TopLeft.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_TopMiddle.DragDelta += new DragDeltaEventHandler(HandleTopMiddleThumb);
			_TopMiddle.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_TopRight.DragDelta += new DragDeltaEventHandler(HandleTopRightThumb);
			_TopRight.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_BottomLeft.DragDelta += new DragDeltaEventHandler(HandleBottomLeftThumb);
			_BottomLeft.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_BottomMiddle.DragDelta += new DragDeltaEventHandler(HandleBottomMiddleThumb);
			_BottomMiddle.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_BottomRight.DragDelta += new DragDeltaEventHandler(HandleBottomRightThumb);
			_BottomRight.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_LeftMiddle.DragDelta += new DragDeltaEventHandler(HandleLeftMiddleThumb);
			_LeftMiddle.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
			_RightMiddle.DragDelta += new DragDeltaEventHandler(HandleRightMiddleThumb);
			_RightMiddle.PreviewMouseUp += new MouseButtonEventHandler(HandleMouseUp);
		}

		private void HandleMouseUp(object sender, MouseButtonEventArgs e)
		{
			if ((_AdornedElement.Height != _OriginalHeight) 
				|| (_AdornedElement.Width != _OriginalWith)
				|| (_AdornedElement.Margin.Left != _OriginalLeft)
				|| (_AdornedElement.Margin.Top != _OriginalTop))
				this.BorderChanged(_AdornedElement);
		}

		public event Action<Border> BorderChanged;

		#region Handle Thumb events
		private void HandleCenterThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			Thickness tMargin = new Thickness();
			tMargin.Left = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Left + args.HorizontalChange);
			tMargin.Top = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Top + args.VerticalChange);

			if ((tAdornedElement.Margin.Left != tMargin.Left) || (tAdornedElement.Margin.Top != tMargin.Top))
				tAdornedElement.Margin = tMargin;
		}

		/// <summary>Handler for resizing from the top-left.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="args"></param>
		private void HandleTopLeftThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height - args.VerticalChange);
			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width - args.HorizontalChange);

			if ((tHeight != tAdornedElement.Height) || (tWidth != tAdornedElement.Width))
			{
				tAdornedElement.Width = tWidth;
				tAdornedElement.Height = tHeight;
				Thickness tMargin = new Thickness();
				tMargin.Left = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Left + args.HorizontalChange);
				tMargin.Top = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Top + args.VerticalChange);
				tAdornedElement.Margin = tMargin;
			}
		}

		private void HandleTopMiddleThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height - args.VerticalChange);

			if (tHeight != tAdornedElement.Height)
			{
				tAdornedElement.Height = tHeight;
				Thickness tMargin = new Thickness();
				tMargin.Left = tAdornedElement.Margin.Left;
				tMargin.Top = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Top + args.VerticalChange);
				tAdornedElement.Margin = tMargin;
			}
		}

		// Handler for resizing from the top-right.
		private void HandleTopRightThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height - args.VerticalChange);
			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width + args.HorizontalChange);

			if ((tHeight != tAdornedElement.Height) || (tWidth != tAdornedElement.Width))
			{
				tAdornedElement.Width = tWidth;
				tAdornedElement.Height = tHeight;

				Thickness tMargin = new Thickness();
				tMargin.Left = tAdornedElement.Margin.Left;
				tMargin.Top = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Top + args.VerticalChange);
				tAdornedElement.Margin = tMargin;
			}
		}

		// Handler for resizing from the bottom-left.
		private void HandleBottomLeftThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width - args.HorizontalChange);
			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height + args.VerticalChange);

			if ((tWidth != tAdornedElement.Width) || (tHeight != tAdornedElement.Height))
			{
				tAdornedElement.Width = tWidth;
				tAdornedElement.Height = tHeight;

				Thickness tMargin = new Thickness();
				tMargin.Left = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Left + args.HorizontalChange);
				tMargin.Top = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Top);
				tAdornedElement.Margin = tMargin;
			}
		}

		// Handler for resizising from bottom-middle
		private void HandleBottomMiddleThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;
			FrameworkElement tParentElement = tAdornedElement.Parent as FrameworkElement;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height + args.VerticalChange);

			if (tHeight != tAdornedElement.Height)
				tAdornedElement.Height = tHeight;
		}

		// Handler for resizing from the bottom-right.
		private void HandleBottomRightThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;
			FrameworkElement tParentElement = tAdornedElement.Parent as FrameworkElement;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tHeight = ChartHelper.GetSnapPosition(tAdornedElement.Height + args.VerticalChange);
			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width + args.HorizontalChange);

			if ((tWidth != tAdornedElement.Width) || (tHeight != tAdornedElement.Height))
			{
				tAdornedElement.Height = tHeight;
				tAdornedElement.Width = tWidth;
			}
		}

		private void HandleLeftMiddleThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width - args.HorizontalChange);

			if (tWidth != tAdornedElement.Width)
			{
				tAdornedElement.Width = tWidth;
				Thickness tMargin = new Thickness();
				tMargin.Left = ChartHelper.GetSnapPosition(tAdornedElement.Margin.Left + args.HorizontalChange);
				tMargin.Top = tAdornedElement.Margin.Top;
				tAdornedElement.Margin = tMargin;
			}
		}

		private void HandleRightMiddleThumb(object sender, DragDeltaEventArgs args)
		{
			FrameworkElement tAdornedElement = this.AdornedElement as FrameworkElement;
			Thumb tHitThumb = sender as Thumb;

			if (tAdornedElement == null || tHitThumb == null) return;

			// Ensure that the Width and Height are properly initialized after the resize.
			EnforceSize(tAdornedElement);

			double tWidth = ChartHelper.GetSnapPosition(tAdornedElement.Width + args.HorizontalChange);

			if (tWidth != tAdornedElement.Width)
				tAdornedElement.Width = tWidth;
		}
		#endregion

		#region Helpers: BuildAdornerCorner, EnforceSize
		/// <summary>Helper method to instantiate the corner Thumbs, set the Cursor property, 
		/// set some appearance properties, and add the elements to the visual tree.
		/// </summary>
		/// <param name="cornerThumb"></param>
		/// <param name="customizedCursor"></param>
		private void BuildAdornerCorner(ref Thumb cornerThumb, Cursor customizedCursor)
		{
			if (cornerThumb != null) return;

			cornerThumb = new Thumb();

			// Set some arbitrary visual characteristics.
			cornerThumb.Cursor = customizedCursor;
			cornerThumb.Height = cornerThumb.Width = 10;
			cornerThumb.Opacity = 0.40;
			cornerThumb.BorderBrush = new SolidColorBrush(Colors.DarkGray);
			cornerThumb.Background = new SolidColorBrush(Colors.MidnightBlue);

			_VisualChildren.Add(cornerThumb);
		}

		/// <summary>This method ensures that the Widths and Heights are initialized.
		/// Sizing to content produces Width and Height values of Double.NaN.
		/// Because this Adorner explicitly resizes, the Width and Height need to be set first.
		/// It also sets the maximum size of the adorned element.
		/// </summary>
		/// <param name="adornedElement"></param>
		private void EnforceSize(FrameworkElement adornedElement)
		{
			if (adornedElement.Width.Equals(Double.NaN))
				adornedElement.Width = adornedElement.DesiredSize.Width;
			if (adornedElement.Height.Equals(Double.NaN))
				adornedElement.Height = adornedElement.DesiredSize.Height;

			FrameworkElement parent = adornedElement.Parent as FrameworkElement;
			if (parent != null)
			{
				adornedElement.MaxHeight = parent.ActualHeight;
				adornedElement.MaxWidth = parent.ActualWidth;
			}
		}
		#endregion

		#region Overrides
		/// <summary>Arrange the Adorners.
		/// </summary>
		/// <param name="finalSize"></param>
		/// <returns></returns>
		protected override Size ArrangeOverride(Size finalSize)
		{
			// tDesiredWidth and tDesiredHeight are the width and height of the element that's being adorned.  
			// These will be used to place the ResizingAdorner at the corners of the adorned element.  
			double tDesiredWidth = AdornedElement.RenderSize.Width; //AdornedElement.DesiredSize.Width;
			double tDesiredHeight = AdornedElement.RenderSize.Height; //AdornedElement.DesiredSize.Height;
			// tAdornerWidth & tAdornerHeight are used for placement as well.
			double tAdornerWidth = this.DesiredSize.Width;
			double tAdornerHeight = this.DesiredSize.Height;

			_LeftMiddle.Arrange(new Rect(-tAdornerWidth / 2, 0, tAdornerWidth, tAdornerHeight));
			_RightMiddle.Arrange(new Rect(tDesiredWidth - tAdornerWidth / 2, 0, tAdornerWidth, tAdornerHeight));
			_TopLeft.Arrange(new Rect(-tAdornerWidth / 2, -tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_TopMiddle.Arrange(new Rect(0, -tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_TopRight.Arrange(new Rect(tDesiredWidth - tAdornerWidth / 2, -tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_BottomLeft.Arrange(new Rect(-tAdornerWidth / 2, tDesiredHeight - tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_BottomMiddle.Arrange(new Rect(0, tDesiredHeight - tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_BottomRight.Arrange(new Rect(tDesiredWidth - tAdornerWidth / 2, tDesiredHeight - tAdornerHeight / 2, tAdornerWidth, tAdornerHeight));
			_CenterThumb.Arrange(new Rect(0, 0, tAdornerWidth, tAdornerHeight));
			// Return the final size.
			return finalSize;
		}

		// Override the VisualChildrenCount and GetVisualChild properties to interface with 
		// the adorner's visual collection.
		protected override int VisualChildrenCount { get { return _VisualChildren.Count; } }
		protected override Visual GetVisualChild(int index) { return _VisualChildren[index]; }
		#endregion
	}
}
