﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Media;

namespace AB3.AppDesigner.Adorners
{
	public class StepDragAdorner: Adorner
	{
		private AdornerLayer _AdornerLayer;
		private Point _Position;
		private string _StepType = null;
		private string _Caption = null;

		public StepDragAdorner(UIElement owner, string stepType, string caption): base(owner)
		{
			_StepType = stepType;
			_Caption = caption;

			_AdornerLayer = AdornerLayer.GetAdornerLayer(owner);
			_AdornerLayer.Add(this);
			owner.DragOver += Owner_DragOver;
		}

		public void Owner_DragOver(object sender, DragEventArgs e)
		{
			_Position = e.GetPosition(base.AdornedElement) + new Vector(10, 10);
			base.InvalidateVisual();
		}

		protected override void OnRender(System.Windows.Media.DrawingContext drawingContext)
		{
			if (this._Position == null || this._Position.X == 0 || this._Position.Y == 0)
				return;
			
			SolidColorBrush tBackgroundBrush;
			SolidColorBrush tBorderBrush = Brushes.Black;
			SolidColorBrush tFontBrush = Brushes.Black;

			if (_StepType == "SingleStep")
			{
				tBackgroundBrush = Brushes.LightBlue;
			}
			else if (_StepType == "FlowStep")
			{
				tBackgroundBrush = Brushes.Gold;
			}
			else
				return;

			FormattedText tFormattedText = new FormattedText(_Caption, System.Globalization.CultureInfo.InvariantCulture, FlowDirection.LeftToRight, new Typeface("Verdanda"), 13, tFontBrush);
			drawingContext.DrawRoundedRectangle(tBackgroundBrush, new Pen(tBorderBrush, 1), 
				new Rect(this._Position, this._Position + new Vector((tFormattedText.Width + 10), (tFormattedText.Height + 10))), 2, 2);

			drawingContext.DrawText(tFormattedText, this._Position + new Vector(5, 5));
			base.OnRender(drawingContext);
		}

		public void Detach()
		{
			_AdornerLayer.Remove(this);
		}
	}
}
