﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Shapes;
using AB3.AppDesigner.Adorners;
using AB3.DataTypes;

namespace AB3.AppDesigner.BaseClasses
{
	public abstract class WireLineDecoratorBase
	{
		#region Fields & Properties
		public FDLWireDefinition Definition { get; private set; }
		public Line View { get; private set; }
		private static LineAdorner _LineAdorner;
		#endregion

		#region Static ctor
		static WireLineDecoratorBase()
		{
			_LineAdorner = null;
		}
		#endregion

		#region ctor
		public WireLineDecoratorBase(FDLWireDefinition wireDefinition)
		{			
			this.Definition = wireDefinition;

			this.View = new Line();
			this.View.Stroke = System.Windows.Media.Brushes.DarkCyan;
			this.View.HorizontalAlignment = HorizontalAlignment.Left;
			this.View.VerticalAlignment = VerticalAlignment.Top;
			this.View.StrokeThickness = 2;

			this.View.MouseDown += View_MouseDown;
		}
		#endregion

		#region Set Start | End PositionOfWire
		protected void SetStartPositionOfWire(FDLWireDefinition wire, Border source, Border target, Line line)
		{
			if (wire.Attributes.ContainsKey("SourceEdge"))
			{
				string tSourceEdge = wire.Attributes["SourceEdge"].Value;
				line.X1 = GetXByEdge(source, tSourceEdge);
				line.Y1 = GetYByEdge(source, tSourceEdge);
			}
			else
			{
				if (source.Margin.Left == target.Margin.Left)
					line.X1 = source.Margin.Left + source.Width / 2;
				else if (source.Margin.Left < target.Margin.Left)
					line.X1 = source.Margin.Left + source.Width;
				else
					line.X1 = source.Margin.Left;

				if (source.Margin.Top == target.Margin.Top)
					line.Y1 = source.Margin.Top + source.Height / 2;
				else if (source.Margin.Top < target.Margin.Top)
					line.Y1 = source.Margin.Top + source.Height;
				else
					line.Y1 = source.Margin.Top;
			}
		}

		protected string SetEndPositionOfWire(FDLWireDefinition wire, Border source, Border target, Line line)
		{
			string tTargetEdge;
			if (wire.Attributes.ContainsKey("TargetEdge"))
			{
				tTargetEdge = wire.Attributes["TargetEdge"].Value;
				line.X2 = GetXByEdge(target, tTargetEdge);
				line.Y2 = GetYByEdge(target, tTargetEdge);
			}
			else
			{
				tTargetEdge = "Left";
				if (source.Margin.Left == target.Margin.Left)
				{
					line.X2 = line.X1;
					if (line.Y2 > line.Y1)
						tTargetEdge = "Top";
					else
						tTargetEdge = "Bottom";
				}
				else if (source.Margin.Left < target.Margin.Left)
					line.X2 = target.Margin.Left;
				else
				{
					line.X2 = target.Margin.Left + target.Width;
					tTargetEdge = "Right";
				}

				if (source.Margin.Top == target.Margin.Top)
				{
					line.Y2 = line.Y1;
					if (line.X2 < line.X1)
						tTargetEdge = "Right";
				}
				else if (source.Margin.Top < target.Margin.Top)
				{
					line.Y2 = target.Margin.Top;
					tTargetEdge = "Top";
				}
				else
				{
					line.Y2 = target.Margin.Top + target.Height;
					tTargetEdge = "Bottom";
				}
			}

			return tTargetEdge;
		}
		#endregion

		#region Get X | Y ByEdge
		private double GetXByEdge(Border border, string edge)
		{
			if ((edge == "Left") || (edge == "TopLeft") || (edge == "BottomLeft"))
				return border.Margin.Left;
			else if ((edge == "Top") || (edge == "Bottom"))
				return border.Margin.Left + border.Width / 2;
			else if ((edge == "Right") || (edge == "TopRight") || (edge == "BottomRight"))
				return border.Margin.Left + border.Width;
			else
				return 0;
		}

		private double GetYByEdge(Border border, string edge)
		{
			if ((edge == "Left") || (edge == "Right"))
				return border.Margin.Top + border.Height / 2;
			else if ((edge == "Top") || (edge == "TopLeft") || (edge == "TopRight"))
				return border.Margin.Top;
			else if ((edge == "Bottom") || (edge == "BottomLeft") || (edge == "BottomRight"))
				return border.Margin.Top + border.Height;
			else
				return 0;
		}
		#endregion
		
		private void View_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (e.ClickCount == 1)
			{
				SelectWireLineDecorator();
				e.Handled = true;
			}
		}

		public void SelectWireLineDecorator()
		{
			AdornerLayer tAdornerLayer = AdornerLayer.GetAdornerLayer(this.View);

			if (_LineAdorner != null)
			{
				tAdornerLayer.Remove(_LineAdorner);
			}

			_LineAdorner = new LineAdorner(this.View);
			tAdornerLayer.Add(_LineAdorner);
			_LineAdorner.StartThumbChanged += LineAdorner_StartThumbChanged;
			_LineAdorner.TargetThumbChanged += LineAdorner_TargetThumbChanged;
			_LineAdorner.ChangeFinished += LineAdorner_ChangeFinished;
		}

		public void RefreshViewBySourcePoint(Point sourcePoint)
		{
			this.View.X1 = sourcePoint.X;
			this.View.Y1 = sourcePoint.Y;
		}
		public void RefreshViewByTargetPoint(Point sourcePoint)
		{
			this.View.X2 = sourcePoint.X;
			this.View.Y2 = sourcePoint.Y;
		}

		protected abstract void LineAdorner_StartThumbChanged(Line line, Point position);
		protected abstract void LineAdorner_TargetThumbChanged(Line line, Point position);
		protected abstract void LineAdorner_ChangeFinished();
	}
}
