﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Media.Effects;
using AB3.AppDesigner.Adorners;
using AB3.AppDesigner.Converters;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class InterfaceDecorator
	{
		#region Static Layout definitions
		private static SolidColorBrush _BackgroundBrush;
		private static SolidColorBrush _BorderBrush;
		private static Thickness _BorderThickness;
		private static DropShadowEffect _ShadowEffect;
		private static CornerRadius _CornerRadius;
		#endregion

		public FDLInterfaceDefinition Definition { get; set; }
		public Border View { get; private set; }
		private BorderAdorner _BorderAdorner = null;

		#region Static ctor
		static InterfaceDecorator()
		{
			_BackgroundBrush = new SolidColorBrush(Colors.LightGreen);
			_BorderBrush = new SolidColorBrush(Colors.Black);
			_BorderThickness = new Thickness(2);
			_CornerRadius = new CornerRadius(8);

			//See: http://msdn.microsoft.com/de-de/library/vstudio/ms748273.aspx
			_ShadowEffect = new DropShadowEffect();
			_ShadowEffect.ShadowDepth = 4;
			_ShadowEffect.Color = Colors.Black;
			_ShadowEffect.Opacity = 0.5;
			_ShadowEffect.BlurRadius = 4;
		}
		#endregion

		public InterfaceDecorator(FDLInterfaceDefinition interfaceDefinition)
		{
			this.Definition = interfaceDefinition;

			#region TextBlock
			TextBlock tTextBlock = new TextBlock();
			tTextBlock.Text = interfaceDefinition.Name;
			tTextBlock.TextWrapping = TextWrapping.Wrap;
			tTextBlock.HorizontalAlignment = HorizontalAlignment.Center;
			tTextBlock.VerticalAlignment = VerticalAlignment.Center;
			#endregion

			this.View = new Border();
			this.View.Child = tTextBlock;
			this.View.BorderBrush = _BorderBrush;
			this.View.Background = _BackgroundBrush;
			this.View.BorderThickness = _BorderThickness;
			this.View.CornerRadius = _CornerRadius;
			this.View.Margin = FDLAttributeConverter.ToPosition(this.Definition.Attributes, 10, 50);
			this.View.HorizontalAlignment = HorizontalAlignment.Left;
			this.View.VerticalAlignment = VerticalAlignment.Top;
			FDLAttributeConverter.SetSize(this.Definition.Attributes, this.View, 100, 25);
			this.View.Effect = _ShadowEffect;
			this.View.MouseDown += View_MouseDown;
		}

		private void View_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (e.ClickCount == 1)
			{
				AdornerLayer tAdornerLayer = AdornerLayer.GetAdornerLayer(this.View);

				if (_BorderAdorner == null)
				{
					_BorderAdorner = new BorderAdorner(this.View);
					_BorderAdorner.BorderChanged += _BorderAdorner_BorderChanged;
					tAdornerLayer.Add(_BorderAdorner);
				}
				else
				{
					tAdornerLayer.Remove(_BorderAdorner);
					_BorderAdorner = null;
				}

				e.Handled = true;
			}
			else if (e.ClickCount == 2)
			{
				Out_ViewDoubleClicked(this.Definition);
				e.Handled = true;
			}
		}

		private void _BorderAdorner_BorderChanged(Border obj)
		{
			Out_ViewChanged(this.Definition, this.View);
		}

		public event Action<FDLInterfaceDefinition, Border> Out_ViewChanged;
		public event Action<FDLInterfaceDefinition> Out_ViewDoubleClicked;
	}
}
