﻿using System;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;
using AB3.AppDesigner.Helpers;

namespace AB3.AppDesigner.Decorators
{
	public class WireArrowDecorator
	{
		#region Static Layout definitions
		private static DropShadowEffect _ShadowEffect;
		#endregion

		#region Instance fields and properties
		private Line _Line;
		public Path View { get; private set; }
		#endregion

		#region Static ctor
		static WireArrowDecorator()
		{
			//See: http://msdn.microsoft.com/de-de/library/vstudio/ms748273.aspx
			_ShadowEffect = new DropShadowEffect();
			_ShadowEffect.ShadowDepth = 4;
			_ShadowEffect.Color = Colors.Black;
			_ShadowEffect.Opacity = 0.5;
			_ShadowEffect.BlurRadius = 4;
		}
		#endregion

		#region Instance ctor
		public WireArrowDecorator(Line line)
		{
			_Line = line;
			this.View = new Path();
			this.View.Data = ChartHelper.GetArrowPathGeometry(line);
			this.View.Fill = System.Windows.Media.Brushes.DarkCyan;
			this.View.Effect = _ShadowEffect;
			this.View.MouseDown += View_MouseDown;
		}		
		#endregion

		#region View-EventHandlers: MouseDown
		private void View_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (e.ClickCount == 1)
			{
				ViewSingleClicked();
				e.Handled = true;
			}
		}
		#endregion

		#region Public Methods: RefreshView
		public void RefreshView()
		{
			PathGeometry tArrowPathGeometry = ChartHelper.GetArrowPathGeometry(_Line);
			this.View.Data = tArrowPathGeometry;
			this.View.Fill = System.Windows.Media.Brushes.DarkCyan;
			this.View.Effect = _ShadowEffect;
		}
		#endregion

		#region Public Events: MouseDown
		public event Action ViewSingleClicked;
		#endregion
	}
}
