﻿using System;
using System.Collections.Generic;
using System.Windows;
using AB3.AppDesigner.BaseClasses;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireDecorator
	{
		#region Sub-Decorators: Arrow, Caption, Point, Line
		public WireArrowDecorator ArrowDecorator { get; private set; }
		public WireCaptionDecorator CaptionDecorator { get; private set; }
		public Dictionary<string, WirePointDecorator> PointDecorators { get; private set; }
		public List<WireLineDecoratorBase> LineDecorators { get; private set; }
		#endregion

		#region Instance ctor & used sub routines
		public WireDecorator(FDLWireDefinition wireDefinition, int orderNumber)
		{
			this.PointDecorators = new Dictionary<string, WirePointDecorator>();
			this.LineDecorators = new List<WireLineDecoratorBase>();

			if (wireDefinition.Attributes.ContainsKey("Point1"))
			{
				FDLAttribute tLastPointAttribute = wireDefinition.Attributes["Point1"];
				WirePointDecorator tWirePointDecorator = new WirePointDecorator(tLastPointAttribute);
				this.PointDecorators.Add(tWirePointDecorator.Name, tWirePointDecorator);
				WireLineSourceToPointDecorator tWireLineSourceToPointDecorator = new WireLineSourceToPointDecorator(wireDefinition, tWirePointDecorator);
				tWireLineSourceToPointDecorator.RefreshViewOfCaptionDecorator += () => this.CaptionDecorator.RefreshView();
				tWireLineSourceToPointDecorator.RefreshViewOfNextDecorators += RefreshViewOfNextDecorators;
				tWireLineSourceToPointDecorator.SourceEdgeChanged += (a, b) => this.Out_SourceEdgeChanged(a, b);
				tWireLineSourceToPointDecorator.TargetPointChanged += (a, b, c) => this.Out_PointChanged(a, b, c);
				this.LineDecorators.Add(tWireLineSourceToPointDecorator);
				this.CaptionDecorator = new WireCaptionDecorator(wireDefinition, orderNumber, tWireLineSourceToPointDecorator.View);
				this.CaptionDecorator.ViewSingleClicked += CaptionDecorator_ViewSingleClicked;
				int tPointNum = 2;
				
				while (wireDefinition.Attributes.ContainsKey(string.Concat("Point", tPointNum)))
				{
					FDLAttribute tNextPointAttribute = wireDefinition.Attributes[string.Concat("Point", tPointNum)];
					tWirePointDecorator = new WirePointDecorator(tNextPointAttribute);
					this.PointDecorators.Add(tWirePointDecorator.Name, tWirePointDecorator);
					WirePointDecorator tSourcePointDecorator = this.PointDecorators[tLastPointAttribute.Name];
					WirePointDecorator tTargetPointDecorator = this.PointDecorators[tNextPointAttribute.Name];
					WireLinePointToPointDecorator tWireLinePointToPointDecorator = new WireLinePointToPointDecorator(wireDefinition, tSourcePointDecorator, tTargetPointDecorator);
					tWireLinePointToPointDecorator.RefreshViewOfNextDecorators += RefreshViewOfNextDecorators;
					tWireLinePointToPointDecorator.RefreshViewOfPreviousDecorators += RefreshViewOfPreviousDecorators;
					tWireLinePointToPointDecorator.SourcePointChanged += (a, b, c) => this.Out_PointChanged(a, b, c);
					tWireLinePointToPointDecorator.TargetPointChanged += (a, b, c) => this.Out_PointChanged(a, b, c);
					this.LineDecorators.Add(tWireLinePointToPointDecorator);
					tLastPointAttribute = tNextPointAttribute;
					tPointNum++;
				}

				WireLinePointToTargetDecorator tWireLinePointToTargetDecorator = new WireLinePointToTargetDecorator(wireDefinition, this.PointDecorators[tLastPointAttribute.Name]);
				tWireLinePointToTargetDecorator.RefreshViewOfArrowDecorator += ( ) => this.ArrowDecorator.RefreshView();
				tWireLinePointToTargetDecorator.RefreshViewOfPreviousDecorators += RefreshViewOfPreviousDecorators;
				tWireLinePointToTargetDecorator.SourcePointChanged += (a, b, c) => this.Out_PointChanged(a, b, c);
				tWireLinePointToTargetDecorator.TargetEdgeChanged += (a, b) => this.Out_TargetEdgeChanged(a, b);
				this.LineDecorators.Add(tWireLinePointToTargetDecorator);
				this.ArrowDecorator = new WireArrowDecorator(tWireLinePointToTargetDecorator.View);
				this.ArrowDecorator.ViewSingleClicked += ArrowDecorator_ViewSingleClicked;
			}
			else
			{
				WireLineSourceToTargetDecorator tWireLineSourceToTargetDecorator = new WireLineSourceToTargetDecorator(wireDefinition);
				tWireLineSourceToTargetDecorator.RefreshViewOfArrowDecorator += () => this.ArrowDecorator.RefreshView();
				tWireLineSourceToTargetDecorator.RefreshViewOfCaptionDecorator += () => this.CaptionDecorator.RefreshView();
				tWireLineSourceToTargetDecorator.SourceEdgeChanged += (a, b) => this.Out_SourceEdgeChanged(a, b);
				tWireLineSourceToTargetDecorator.TargetEdgeChanged += (a, b) => this.Out_TargetEdgeChanged(a, b);
				this.LineDecorators.Add(tWireLineSourceToTargetDecorator);
				this.CaptionDecorator = new WireCaptionDecorator(wireDefinition, orderNumber, tWireLineSourceToTargetDecorator.View);
				this.CaptionDecorator.ViewSingleClicked += CaptionDecorator_ViewSingleClicked;
				this.ArrowDecorator = new WireArrowDecorator(tWireLineSourceToTargetDecorator.View);
				this.ArrowDecorator.ViewSingleClicked += ArrowDecorator_ViewSingleClicked;
			}
		}
		#endregion
		
		#region MouseDown EventHandlers of Arrow & Caption Decorator
		private void ArrowDecorator_ViewSingleClicked()
		{
			this.LineDecorators[this.LineDecorators.Count - 1].SelectWireLineDecorator(); // WireEnd belongs to last segment
		}

		private void CaptionDecorator_ViewSingleClicked()
		{
			this.LineDecorators[0].SelectWireLineDecorator(); // WireCaption belongs allways first wire segment
		}
		#endregion

		#region RefreshView Event Handlers
		private void RefreshViewOfNextDecorators(WireLineDecoratorBase sender, Point point)
		{
			int tNextIndex = this.LineDecorators.IndexOf(sender) + 1;
			string tPointDecoratorName = string.Concat("Point", tNextIndex);
			WirePointDecorator tWirePointDecorator = this.PointDecorators[tPointDecoratorName];
			tWirePointDecorator.RefreshView(point);

			WireLineDecoratorBase tWireLineDecorator = this.LineDecorators[tNextIndex];
			tWireLineDecorator.RefreshViewBySourcePoint(point);

			if (tWireLineDecorator is WireLinePointToTargetDecorator) // The Arrow has to be refreshed too
				this.ArrowDecorator.RefreshView();
		}

		private void RefreshViewOfPreviousDecorators(WireLineDecoratorBase sender, Point point)
		{
			int tPreviousIndex = this.LineDecorators.IndexOf(sender);
			string tPointDecoratorName = string.Concat("Point", tPreviousIndex);
			WirePointDecorator tWirePointDecorator = this.PointDecorators[tPointDecoratorName];
			tWirePointDecorator.RefreshView(point);

			WireLineDecoratorBase tWireLineDecorator = this.LineDecorators[tPreviousIndex - 1];
			tWireLineDecorator.RefreshViewByTargetPoint(point);

			if (tWireLineDecorator is WireLineSourceToPointDecorator) // The Caption has to be refreshed too
				this.CaptionDecorator.RefreshView();
		}
		#endregion
		
		public event Action<FDLWireDefinition, string> Out_SourceEdgeChanged;
		public event Action<FDLWireDefinition, string> Out_TargetEdgeChanged;
		public event Action<FDLWireDefinition, string, Point> Out_PointChanged;
	}
}
