﻿using System;
using System.Windows;
using System.Windows.Controls;
using AB3.AppDesigner.BaseClasses;
using AB3.AppDesigner.DataTypes;
using AB3.AppDesigner.Helpers;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireLinePointToTargetDecorator : WireLineDecoratorBase
	{
		private WirePointDecorator _SourcePointDecorator;
		private string _NewTargetEdgeValue;
		private Point _NewSourcePointPosition;

		public WireLinePointToTargetDecorator(FDLWireDefinition wireDefinition, WirePointDecorator pointDecorator): base(wireDefinition)
		{
			_SourcePointDecorator = pointDecorator;
			Border tSource = pointDecorator.View;
			Border tTarget = ChartHelper.GetTargetBorder(wireDefinition);

			if (tTarget != null)
			{
				base.View.X1 = tSource.Margin.Left + tSource.Width / 2;
				base.View.Y1 = tSource.Margin.Top + tSource.Height / 2;
				string tTargetEdge = SetEndPositionOfWire(wireDefinition, tSource, tTarget, base.View);
			}
		}

		protected override void LineAdorner_StartThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			_NewSourcePointPosition = ChartHelper.GetSnapPoint(position);
			line.X1 = _NewSourcePointPosition.X;
			line.Y1 = _NewSourcePointPosition.Y;
			RefreshViewOfPreviousDecorators(this, _NewSourcePointPosition);
		}

		protected override void LineAdorner_TargetThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			Border tTarget = ChartHelper.GetTargetBorder(this.Definition);
			EdgePosition tEdgePosition = ChartHelper.GetEdgePointByPosition(tTarget, position);

			if (tEdgePosition != null)
			{
				line.X2 = tEdgePosition.X;
				line.Y2 = tEdgePosition.Y;
				_NewTargetEdgeValue = tEdgePosition.Caption;
				RefreshViewOfArrowDecorator();
			}
		}

		protected override void LineAdorner_ChangeFinished()
		{
			if ((_NewSourcePointPosition.X != 0) || (_NewSourcePointPosition.Y != 0))
			{
				SourcePointChanged(base.Definition, _SourcePointDecorator.Name, _NewSourcePointPosition);
			}
			else if (!string.IsNullOrEmpty(_NewTargetEdgeValue))
			{
				if (!base.Definition.Attributes.ContainsKey("TargetEdge"))
				{
					TargetEdgeChanged(base.Definition, _NewTargetEdgeValue);
				}
				else if (base.Definition.Attributes["TargetEdge"].Value != _NewTargetEdgeValue)
				{
					TargetEdgeChanged(base.Definition, _NewTargetEdgeValue);
				}
			}
		}

		public event Action RefreshViewOfArrowDecorator;
		public event Action<WireLineDecoratorBase, Point> RefreshViewOfPreviousDecorators;
		public event Action<FDLWireDefinition, string, Point> SourcePointChanged;
		public event Action<FDLWireDefinition, string> TargetEdgeChanged;
	}
}
