﻿using System;
using System.Windows;
using System.Windows.Controls;
using AB3.AppDesigner.BaseClasses;
using AB3.AppDesigner.DataTypes;
using AB3.AppDesigner.Helpers;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireLineSourceToPointDecorator : WireLineDecoratorBase
	{
		private string _NewSourceEdgeValue;
		private Point _NewTargetPointPosition;
		private WirePointDecorator _TargetPointDecorator;

		public WireLineSourceToPointDecorator(FDLWireDefinition wireDefinition, WirePointDecorator pointDecorator): base(wireDefinition)
		{
			_TargetPointDecorator = pointDecorator;
			_NewSourceEdgeValue = null;
			_NewTargetPointPosition = new Point();

			Border tSource = ChartHelper.GetSourceBorder(wireDefinition);
			Border tTarget = pointDecorator.View;

			if (tSource != null)
			{
				base.SetStartPositionOfWire(wireDefinition, tSource, tTarget, base.View);

				base.View.X2 = tTarget.Margin.Left + tTarget.Width / 2;
				base.View.Y2 = tTarget.Margin.Top + tTarget.Height / 2;
			}
		}

		protected override void LineAdorner_StartThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			Border tSource = ChartHelper.GetSourceBorder(this.Definition);
			EdgePosition tEdgePosition = ChartHelper.GetEdgePointByPosition(tSource, position);

			if (tEdgePosition != null)
			{
				line.X1 = tEdgePosition.X;
				line.Y1 = tEdgePosition.Y;
				_NewSourceEdgeValue = tEdgePosition.Caption;
				RefreshViewOfCaptionDecorator();
			}
		}

		protected override void LineAdorner_TargetThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			_NewTargetPointPosition = ChartHelper.GetSnapPoint(position);
			line.X2 = _NewTargetPointPosition.X;
			line.Y2 = _NewTargetPointPosition.Y;
			RefreshViewOfNextDecorators(this, _NewTargetPointPosition);
			RefreshViewOfCaptionDecorator();
		}

		protected override void LineAdorner_ChangeFinished()
		{
			if (!string.IsNullOrEmpty(_NewSourceEdgeValue))
			{
				if (!base.Definition.Attributes.ContainsKey("SourceEdge"))
				{
					SourceEdgeChanged(base.Definition, _NewSourceEdgeValue);
				}
				else if (base.Definition.Attributes["SourceEdge"].Value != _NewSourceEdgeValue)
				{
					SourceEdgeChanged(base.Definition, _NewSourceEdgeValue);
				}
			}
			else if ((_NewTargetPointPosition.X != 0) || (_NewTargetPointPosition.Y != 0))
			{
				TargetPointChanged(base.Definition, _TargetPointDecorator.Name, _NewTargetPointPosition);
			}
		}

		public event Action RefreshViewOfCaptionDecorator;
		public event Action<WireLineDecoratorBase, Point> RefreshViewOfNextDecorators;
		public event Action<FDLWireDefinition, string> SourceEdgeChanged;
		public event Action<FDLWireDefinition, string, Point> TargetPointChanged;
	}
}
