﻿using System;
using System.Windows;
using System.Windows.Controls;
using AB3.AppDesigner.BaseClasses;
using AB3.AppDesigner.DataTypes;
using AB3.AppDesigner.Helpers;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireLineSourceToTargetDecorator : WireLineDecoratorBase
	{
		private string _NewSourceEdgeValue;
		private string _NewTargetEdgeValue;

		public WireLineSourceToTargetDecorator(FDLWireDefinition wireDefinition): base(wireDefinition)
		{
			_NewSourceEdgeValue = null;
			_NewTargetEdgeValue = null;
			Border tSource = ChartHelper.GetSourceBorder(wireDefinition);
			Border tTarget = ChartHelper.GetTargetBorder(wireDefinition);

			if ((tSource != null) && (tTarget != null))
			{
				base.SetStartPositionOfWire(wireDefinition, tSource, tTarget, base.View);
				string tTargetEdge = base.SetEndPositionOfWire(wireDefinition, tSource, tTarget, base.View);
			}
		}

		protected override void LineAdorner_StartThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			Border tSource = ChartHelper.GetSourceBorder(this.Definition);
			EdgePosition tEdgePosition = ChartHelper.GetEdgePointByPosition(tSource, position);

			if (tEdgePosition != null)
			{
				line.X1 = tEdgePosition.X;
				line.Y1 = tEdgePosition.Y;
				_NewSourceEdgeValue = tEdgePosition.Caption;
				RefreshViewOfCaptionDecorator();
			}
		}

		protected override void LineAdorner_TargetThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			Border tTarget = ChartHelper.GetTargetBorder(this.Definition);
			EdgePosition tEdgePosition = ChartHelper.GetEdgePointByPosition(tTarget, position);

			if (tEdgePosition != null)
			{
				line.X2 = tEdgePosition.X;
				line.Y2 = tEdgePosition.Y;
				_NewTargetEdgeValue = tEdgePosition.Caption;
				RefreshViewOfArrowDecorator();
				RefreshViewOfCaptionDecorator();
			}
		}

		protected override void LineAdorner_ChangeFinished()
		{
			if (!string.IsNullOrEmpty(_NewSourceEdgeValue))
			{
				if (!base.Definition.Attributes.ContainsKey("SourceEdge"))
				{
					SourceEdgeChanged(base.Definition, _NewSourceEdgeValue);
				}
				else if (base.Definition.Attributes["SourceEdge"].Value != _NewSourceEdgeValue)
				{
					SourceEdgeChanged(base.Definition, _NewSourceEdgeValue);
				}
			}
			else if (!string.IsNullOrEmpty(_NewTargetEdgeValue))
			{
				if (!base.Definition.Attributes.ContainsKey("TargetEdge"))
				{
					TargetEdgeChanged(base.Definition, _NewTargetEdgeValue);
				}
				else if (base.Definition.Attributes["TargetEdge"].Value != _NewTargetEdgeValue)
				{
					TargetEdgeChanged(base.Definition, _NewTargetEdgeValue);
				}
			}
		}

		public event Action RefreshViewOfArrowDecorator;
		public event Action RefreshViewOfCaptionDecorator;
		public event Action<FDLWireDefinition, string> SourceEdgeChanged;
		public event Action<FDLWireDefinition, string> TargetEdgeChanged;
	}
}
