﻿using System.Windows;
using System.Windows.Controls;
using AB3.AppDesigner.Converters;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WirePointDecorator
	{
		#region Static Layout definitions
		private static Thickness _BorderThickness;
		private static CornerRadius _BorderCornerRadius;
		#endregion

		#region Instance fields & properties
		/// <summary>Name of the point. For example Point4711
		/// </summary>
		public string Name
		{
			get { return _Definition.Name; }
		}

		/// <summary>Attribute definition of the point in FDL notation
		/// </summary>
		private FDLAttribute _Definition;

		/// <summary>The view of the point (Border)
		/// </summary>
		public Border View { get; private set; }
		#endregion

		#region Static ctor
		static WirePointDecorator()
		{
			_BorderThickness = new Thickness(1);
			_BorderCornerRadius = new CornerRadius(3);
		}
		#endregion

		#region Instance ctor
		public WirePointDecorator(FDLAttribute definition)
		{
			_Definition = definition;
			this.View = new Border();
			this.View.BorderBrush = System.Windows.Media.Brushes.Black;
			this.View.BorderThickness = _BorderThickness;
			this.View.CornerRadius = _BorderCornerRadius;
			this.View.Margin = FDLAttributeConverter.ToPointPosition(definition, 10, 10, -2.5);
			this.View.HorizontalAlignment = HorizontalAlignment.Left;
			this.View.VerticalAlignment = VerticalAlignment.Top;
			this.View.Width = 5;
			this.View.Height = 5;
		}
		#endregion

		#region Public Methods: RefreshView
		public void RefreshView(double x, double y)
		{
			Thickness tMargin = new Thickness();
			tMargin.Left = x - 2.5;
			tMargin.Top = y - 2.5;
			this.View.Margin = tMargin;
		}

		public void RefreshView(Point point)
		{
			RefreshView(point.X, point.Y);
		}
		#endregion
	}
}
