﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading;
using AB3.AppDesigner.DataTypes;

namespace AB3.AppDesigner.Logic
{
	public class AppConfigHandler
	{
		private object _Lock = new object();
		private string _AppConfigPathFileName;
		private string[] _AppConfigLines;
		private Dictionary<string, int> _AppConfigMap;

		public AppConfig GetAppConfig()
		{
			string tLocation = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
			this._AppConfigPathFileName = Path.Combine(tLocation, "AB3.AppDesigner.cfg");

			this._AppConfigLines = File.ReadAllLines(this._AppConfigPathFileName);
			this._AppConfigMap = new Dictionary<string, int>();

			for (int i = 0; i < this._AppConfigLines.Length; i++)
			{
				if ((!this._AppConfigLines[i].StartsWith("//")) && ((this._AppConfigLines[i]).Trim().Length != 0))
				{
					string[] tLineParts = this._AppConfigLines[i].Split(new string[] { ":" }, StringSplitOptions.None);
					this._AppConfigMap.Add(tLineParts[0].Trim(), i);
				}
			}

			Dictionary<string, string> tConfigValues = this.GetConfigValues(this._AppConfigLines, this._AppConfigMap);
			AppConfig tAppConfig = new AppConfig(tConfigValues);
			tAppConfig.PropertyChanged += tAppConfig_PropertyChanged;

			return tAppConfig;
		}

		private void tAppConfig_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
		{
			if (this._AppConfigMap.ContainsKey(e.PropertyName))
			{
				int tLineId = this._AppConfigMap[e.PropertyName];
				string tLine = this._AppConfigLines[tLineId];

				string tValueComment = tLine.Replace(e.PropertyName + ":", string.Empty);
				string[] tLineParts2 = tValueComment.Split(new string[] { "//" }, StringSplitOptions.None);
				string tComment = tLineParts2[1].Trim();

				AppConfig tAppConfig = (AppConfig)sender;
				string tNewLine = string.Format("{0}: {1} // {2}", e.PropertyName, tAppConfig.GetValueAsString(e.PropertyName), tComment);
				this._AppConfigLines[tLineId] = tNewLine;

				var thread = new Thread(() => WriteAppConfigValues());
				thread.Start();
			}
		}

		private void WriteAppConfigValues()
		{
			lock (this._Lock)
			{
				using (StreamWriter tStreamWriter = new StreamWriter(this._AppConfigPathFileName, false))
				{
					foreach (string tLine in this._AppConfigLines)
					{
						tStreamWriter.WriteLine(tLine);
					}
					tStreamWriter.Close();
				}
			}
		}

		private Dictionary<string, string> GetConfigValues(string[] appConfigText, Dictionary<string, int> appConfigMap)
		{
			Dictionary<string, string> tConfigValues = new Dictionary<string, string>();
			foreach (string tParameter in appConfigMap.Keys)
			{
				int tRowID = appConfigMap[tParameter];
				string tLine = appConfigText[tRowID];

				string tValueComment = tLine.Replace(tParameter + ":", string.Empty);
				string[] tValueParts = tValueComment.Split(new string[] { "//" }, StringSplitOptions.None);
				tConfigValues.Add(tParameter, tValueParts[0].Trim());
			}

			return tConfigValues;
		}
	}
}
