﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AB3.AppDesigner.DataTypes;
using AB3.AppDesigner.EditWindows;
using AB3.DataTypes;

namespace AB3.AppDesigner.Logic
{
	public class EditWireDefinition
	{
		private FDLWireDefinition _WireDefinition;
		private EditWireWindow _EditWindow;

		public void In_Input(FDLWireDefinition wireDefinition)
		{
			if (!AppDefinitionCache.Value.FlowDefinitions.ContainsKey(FlowDefinitionNameCache.Value))
				return;

			string tPreviousSourceStep = wireDefinition.Source.Step;
			string tPreviousSourcePin = wireDefinition.Source.Pin;
			string tPreviousTargetStep = wireDefinition.Target.Step;
			string tPreviousTargetPin = wireDefinition.Target.Pin;

			_WireDefinition = wireDefinition;
			_EditWindow = new EditWireWindow();
			if (wireDefinition.Attributes.ContainsKey("Caption"))
				_EditWindow.textBoxCaption.Text = wireDefinition.Attributes["Caption"].Value;
			else
				_EditWindow.textBoxCaption.Text = string.Empty;

			if (wireDefinition.ToString() != ".->.")
				_EditWindow.textBoxDefinition.Text = wireDefinition.ToString();
			else
				_EditWindow.textBoxDefinition.Text = string.Empty;

			#region Interfaces
			List<string> tSourceInterfaces = new List<string>();
			List<string> tTargetInterfaces = new List<string>();
			foreach (FDLInterfaceDefinition tInterface in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Interfaces.Collection.Values)
			{
				if (tInterface.Direction == Directions.In)
					tSourceInterfaces.Add(tInterface.Name);
				else if (tInterface.Direction == Directions.Out)
					tTargetInterfaces.Add(tInterface.Name);
			}

			if (tSourceInterfaces.Count == 0)
			{
				_EditWindow.tabItemSourceInterfaces.Visibility = System.Windows.Visibility.Collapsed;
				_EditWindow.tabItemSourceFlows.IsSelected = true;
			}
			else
			{
				tSourceInterfaces.Sort();
				_EditWindow.listBoxSourceInterfaces.ItemsSource = tSourceInterfaces;
				_EditWindow.listBoxSourceInterfaces.MouseDoubleClick += listBoxSourceInterfaces_MouseDoubleClick;
			}

			if (tTargetInterfaces.Count == 0)
			{
				_EditWindow.tabItemTargetInterfaces.Visibility = System.Windows.Visibility.Collapsed;
				_EditWindow.tabItemTargetFlows.IsSelected = true;
			}
			else
			{
				tTargetInterfaces.Sort();
				_EditWindow.listBoxTargetInterfaces.ItemsSource = tTargetInterfaces;
				_EditWindow.listBoxTargetInterfaces.MouseDoubleClick += listBoxTargetInterfaces_MouseDoubleClick;
			}
			#endregion

			#region Flows & Steps
			List<string> tSourceFlows = new List<string>();
			List<string> tSourceSteps = new List<string>();
			List<string> tTargetFlows = new List<string>();
			List<string> tTargetSteps = new List<string>();
			foreach (FDLStepDefinition tStepDefinition in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Steps.Collection.Values)
			{
				if (tStepDefinition.IsSubFlowStep)
				{
					tSourceFlows.Add(tStepDefinition.CustomName);
					tTargetFlows.Add(tStepDefinition.CustomName);
				}
				else
				{
					tSourceSteps.Add(tStepDefinition.CustomName);
					tTargetSteps.Add(tStepDefinition.CustomName);
				}
			}

			if (tSourceFlows.Count == 0)
			{				
				if (_EditWindow.tabItemSourceFlows.IsSelected)
					_EditWindow.tabItemSourceSteps.IsSelected = true;
				_EditWindow.tabItemSourceFlows.Visibility = System.Windows.Visibility.Collapsed;
			}
			else
			{
				tSourceFlows.Sort();
				_EditWindow.listBoxSourceFlowSteps.ItemsSource = tSourceFlows;
				_EditWindow.listBoxSourceFlowSteps.SelectionChanged += listBoxSourceFlowSteps_SelectionChanged;
				_EditWindow.listBoxSourceFlowSteps.MouseDoubleClick += listBoxSourceFlowSteps_MouseDoubleClick;
				_EditWindow.listBoxSourceFlowStepInterfaces.MouseDoubleClick += listBoxSourceFlowStepInterfaces_MouseDoubleClick;
			}

			if (tTargetFlows.Count == 0)
			{
				if (_EditWindow.tabItemTargetFlows.IsSelected)
					_EditWindow.tabItemTargetSteps.IsSelected = true;
				_EditWindow.tabItemTargetFlows.Visibility = System.Windows.Visibility.Collapsed;
			}
			else
			{
				tTargetFlows.Sort();
				_EditWindow.listBoxTargetFlowSteps.ItemsSource = tTargetFlows;
				_EditWindow.listBoxTargetFlowSteps.SelectionChanged += listBoxTargetFlowSteps_SelectionChanged;
				_EditWindow.listBoxTargetFlowSteps.MouseDoubleClick += listBoxTargetFlowSteps_MouseDoubleClick;
				_EditWindow.listBoxTargetFlowStepInterfaces.MouseDoubleClick += listBoxTargetFlowStepInterfaces_MouseDoubleClick;
			}

			if (tSourceSteps.Count == 0)
				_EditWindow.tabItemSourceSteps.Visibility = System.Windows.Visibility.Collapsed;
			else
			{
				tSourceSteps.Sort();
				_EditWindow.listBoxSourceSingleSteps.ItemsSource = tSourceSteps;
				_EditWindow.listBoxSourceSingleSteps.SelectionChanged += listBoxSourceSteps_SelectionChanged;
				_EditWindow.listBoxSourceSingleSteps.MouseDoubleClick += listBoxSourceSteps_MouseDoubleClick;
				_EditWindow.listBoxSourceSingleStepPins.MouseDoubleClick += listBoxSourceStepPins_MouseDoubleClick;
			}

			if (tTargetSteps.Count == 0)
				_EditWindow.tabItemTargetSteps.Visibility = System.Windows.Visibility.Collapsed;
			else
			{
				tTargetSteps.Sort();
				_EditWindow.listBoxTargetSingleSteps.ItemsSource = tTargetSteps;
				_EditWindow.listBoxTargetSingleSteps.SelectionChanged += listBoxTargetSingleSteps_SelectionChanged;
				_EditWindow.listBoxTargetSingleSteps.MouseDoubleClick += listBoxTargetSingleSteps_MouseDoubleClick;
				_EditWindow.listBoxTargetSingleStepPins.MouseDoubleClick += listBoxTargetSingleStepPins_MouseDoubleClick;
			}
			#endregion

			_EditWindow.Owner = MainWindow.Instance;
			_EditWindow.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;

			bool? tResponse = _EditWindow.ShowDialog();
			if ((tResponse.HasValue) && (tResponse.Value == true)) // OK Button
			{
				if (_EditWindow.textBoxDefinition.Text != wireDefinition.ToString()) // There was something entered in the textbox => Parse text
				{
					string[] tLineParts = _EditWindow.textBoxDefinition.Text.Trim().Split(new string[] { "->" }, StringSplitOptions.None);
					if (tLineParts.Length == 1)
					{
						wireDefinition.Source.Step = string.Empty;
						wireDefinition.Source.Pin = string.Empty;
						wireDefinition.Target.Step = string.Empty;
						wireDefinition.Target.Pin = string.Empty;
					}
					else
					{
						string[] tSourceParts = tLineParts[0].Split(new string[] { "." }, StringSplitOptions.None);
						if (tSourceParts.Length == 1)
						{
							wireDefinition.Source.Step = tSourceParts[0];
							wireDefinition.Source.Pin = string.Empty;
						}
						else
						{
							wireDefinition.Source.Step = tSourceParts[0].Trim();
							wireDefinition.Source.Pin = tSourceParts[1].Trim();
						}
						string[] tTargetParts = tLineParts[1].Split(new string[] { "." }, StringSplitOptions.None);
						if (tTargetParts.Length == 1)
						{
							wireDefinition.Target.Step = tSourceParts[0];
							wireDefinition.Target.Pin = string.Empty;
						}
						else
						{
							wireDefinition.Target.Step = tTargetParts[0].Trim();
							wireDefinition.Target.Pin = tTargetParts[1].Trim();
						}
					}
				}

				Out_Output(wireDefinition);
			}
			else // Cancel Button => Restore previous values
			{
				wireDefinition.Source.Step = tPreviousSourceStep;
				wireDefinition.Source.Pin = tPreviousSourcePin;
				wireDefinition.Target.Step = tPreviousTargetStep;
				wireDefinition.Target.Pin = tPreviousTargetPin;
			}

			_EditWindow = null;
			_WireDefinition = null;
		}

		public event Action<FDLWireDefinition> Out_Output;

		#region UI-Events from Target Step Tab
		private void listBoxTargetSingleStepPins_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxTargetSingleSteps.SelectedItem != null)
				_WireDefinition.Target.Step = _EditWindow.listBoxTargetSingleSteps.SelectedItem.ToString();
			else
				_WireDefinition.Target.Step = string.Empty;

			if (_EditWindow.listBoxTargetSingleStepPins.SelectedItem != null)
				_WireDefinition.Target.Pin = _EditWindow.listBoxTargetSingleStepPins.SelectedItem.ToString();
			else
				_WireDefinition.Target.Pin = string.Empty;

			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxTargetSingleSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxTargetSingleSteps.SelectedItem != null)
				_WireDefinition.Target.Step = _EditWindow.listBoxTargetSingleSteps.SelectedItem.ToString();
			else
				_WireDefinition.Target.Step = string.Empty;
			_WireDefinition.Target.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxTargetSingleSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			string tCustomName = _EditWindow.listBoxTargetSingleSteps.SelectedItem.ToString();
			string tTypeName = GetStepTypeNameByCustomName(tCustomName, false);

			List<string> tTargetStepInterfaces = new List<string>();
			foreach (FDLFlowDefinition tFlowDefinition in AppDefinitionCache.Value.FlowDefinitions.Values)
			{
				foreach (FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
				{
					if ((!tStepDefinition.IsSubFlowStep) && (tStepDefinition.TypeName == tTypeName))
					{
						foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
						{
							if (tWireDefinition.Target.Step == tStepDefinition.CustomName)
								tTargetStepInterfaces.Add(tWireDefinition.Target.Pin);
						}
					}
				}
			}

			tTargetStepInterfaces = tTargetStepInterfaces.Distinct().ToList();
			tTargetStepInterfaces.Sort();

			_EditWindow.listBoxTargetSingleStepPins.ItemsSource = tTargetStepInterfaces.Distinct().ToList();
		}
		#endregion

		#region UI-Events from Target Flows tab
		private void listBoxTargetFlowStepInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxTargetFlowSteps.SelectedItem != null)
				_WireDefinition.Target.Step = _EditWindow.listBoxTargetFlowSteps.SelectedItem.ToString();
			else
				_WireDefinition.Target.Step = string.Empty;

			if (_EditWindow.listBoxTargetFlowStepInterfaces.SelectedItem != null)
				_WireDefinition.Target.Pin = _EditWindow.listBoxTargetFlowStepInterfaces.SelectedItem.ToString();
			else
				_WireDefinition.Target.Pin = string.Empty;

			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxTargetFlowSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxTargetFlowSteps.SelectedItem != null)
				_WireDefinition.Target.Step = _EditWindow.listBoxTargetFlowSteps.SelectedItem.ToString();
			else
				_WireDefinition.Target.Step = string.Empty;
			_WireDefinition.Target.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxTargetFlowSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (_EditWindow.listBoxTargetFlowSteps.SelectedItem != null)
			{
				string tCustomName = _EditWindow.listBoxTargetFlowSteps.SelectedItem.ToString();
				string tTypeName = GetStepTypeNameByCustomName(tCustomName, true);
				List<string> tTargetFlowInterfaces = new List<string>(this.GetInterfacesByTypeName(tTypeName, Directions.In));

				_EditWindow.listBoxTargetFlowStepInterfaces.ItemsSource = tTargetFlowInterfaces;
			}
			else
			{
				_EditWindow.listBoxTargetFlowStepInterfaces.ItemsSource = null;
				_EditWindow.listBoxTargetFlowStepInterfaces.Items.Clear();
			}
		}
		#endregion

		#region UI-Events from Target Interfaces tab
		private void listBoxTargetInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			_WireDefinition.Target.Step = "Out";
			if (_EditWindow.listBoxTargetInterfaces.SelectedItem != null)
				_WireDefinition.Target.Pin = _EditWindow.listBoxTargetInterfaces.SelectedItem.ToString();
			else
				_WireDefinition.Target.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}
		#endregion

		#region UI-Events from Source Step Tab
		private void listBoxSourceSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			string tCustomName = _EditWindow.listBoxSourceSingleSteps.SelectedItem.ToString();
			string tTypeName = GetStepTypeNameByCustomName(tCustomName, false);

			List<string> tSourceStepInterfaces = new List<string>();
			foreach (FDLFlowDefinition tFlowDefinition in AppDefinitionCache.Value.FlowDefinitions.Values)
			{
				foreach (FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
				{
					if ((!tStepDefinition.IsSubFlowStep) && (tStepDefinition.TypeName == tTypeName))
					{
						foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
						{
							if (tWireDefinition.Source.Step == tStepDefinition.CustomName)
								tSourceStepInterfaces.Add(tWireDefinition.Source.Pin);
						}
					}
				}
			}

			tSourceStepInterfaces = tSourceStepInterfaces.Distinct().ToList();
			tSourceStepInterfaces.Sort();

			_EditWindow.listBoxSourceSingleStepPins.ItemsSource = tSourceStepInterfaces;
		}

		private void listBoxSourceSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxSourceSingleSteps.SelectedItem != null)
				_WireDefinition.Source.Step = _EditWindow.listBoxSourceSingleSteps.SelectedItem.ToString();
			else
				_WireDefinition.Source.Step = string.Empty;
			_WireDefinition.Source.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxSourceStepPins_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxSourceSingleSteps.SelectedItem != null)
				_WireDefinition.Source.Step = _EditWindow.listBoxSourceSingleSteps.SelectedItem.ToString();
			else
				_WireDefinition.Source.Step = string.Empty;

			if (_EditWindow.listBoxSourceSingleStepPins.SelectedItem != null)
				_WireDefinition.Source.Pin = _EditWindow.listBoxSourceSingleStepPins.SelectedItem.ToString();
			else
				_WireDefinition.Source.Pin = string.Empty;

			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}
		#endregion

		#region UI events from Source Flows tab
		private void listBoxSourceFlowStepInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxSourceFlowSteps.SelectedItem != null)
				_WireDefinition.Source.Step = _EditWindow.listBoxSourceFlowSteps.SelectedItem.ToString();
			else
				_WireDefinition.Source.Step = string.Empty;

			if (_EditWindow.listBoxSourceFlowStepInterfaces.SelectedItem != null)
				_WireDefinition.Source.Pin = _EditWindow.listBoxSourceFlowStepInterfaces.SelectedItem.ToString();
			else
				_WireDefinition.Source.Pin = string.Empty;

			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}

		private void listBoxSourceFlowSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (_EditWindow.listBoxSourceFlowSteps.SelectedItem != null)
			{
				string tCustomName = _EditWindow.listBoxSourceFlowSteps.SelectedItem.ToString();
				string tTypeName = GetStepTypeNameByCustomName(tCustomName, true);
				List<string> tSourceFlowInterfaces = new List<string>(this.GetInterfacesByTypeName(tTypeName, Directions.Out));

				_EditWindow.listBoxSourceFlowStepInterfaces.ItemsSource = tSourceFlowInterfaces;
			}
			else
			{
				_EditWindow.listBoxSourceFlowStepInterfaces.ItemsSource = null;
				_EditWindow.listBoxSourceFlowStepInterfaces.Items.Clear();
			}
		}

		private void listBoxSourceFlowSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (_EditWindow.listBoxSourceFlowSteps.SelectedItem != null)
				_WireDefinition.Source.Step = _EditWindow.listBoxSourceFlowSteps.SelectedItem.ToString();
			else
				_WireDefinition.Source.Step = string.Empty;
			_WireDefinition.Source.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}
		#endregion

		#region UI-Events from Source Interfaces tab
		private void listBoxSourceInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{			
			_WireDefinition.Source.Step = "In";
			if (_EditWindow.listBoxSourceInterfaces.SelectedItem != null)
				_WireDefinition.Source.Pin = _EditWindow.listBoxSourceInterfaces.SelectedItem.ToString();
			else
				_WireDefinition.Source.Pin = string.Empty;
			_EditWindow.textBoxDefinition.Text = _WireDefinition.ToString();
		}
		#endregion

		#region Helper functions: GetInterfacesByTypeName, GetStepTypeNameByCustomName
		private string[] GetInterfacesByTypeName(string typeName, Directions interfaceDirection)
		{
			List<string> tReturn = new List<string>();

			if (AppDefinitionCache.Value.FlowDefinitions.ContainsKey(typeName))
			{
				foreach (FDLInterfaceDefinition tInterfaceDefinition in AppDefinitionCache.Value.FlowDefinitions[typeName].Interfaces.Collection.Values)
				{
					if (tInterfaceDefinition.Direction == interfaceDirection)
						tReturn.Add(tInterfaceDefinition.Name);
				}
			}

			return tReturn.ToArray();
		}

		private string GetStepTypeNameByCustomName(string customName, bool isSubFlowStep)
		{
			string tReturn = null;

			foreach (FDLStepDefinition tStepDefinition in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Steps.Collection.Values)
			{
				if ((tStepDefinition.IsSubFlowStep == isSubFlowStep) && (tStepDefinition.CustomName == customName))
				{
					tReturn = tStepDefinition.TypeName;
					break;
				}
			}

			return tReturn;
		}
		#endregion
	}
}
