﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using AB3.Interfaces;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;

namespace AB3.AppDesigner.Presenters
{
	public class PartsPresenter: IInitialize
	{
		public void In_Input()
		{
			if (MainWindow.Instance.listViewParts.CheckAccess())
			{
				List<string> tComponents = new List<string>();

				foreach (AB3.DataTypes.FDLFlowDefinition tFlowDefinition in AppDefinitionCache.Value.FlowDefinitions.Values)
				{
					foreach (AB3.DataTypes.FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
					{
						string tStepName;
						if (tStepDefinition.IsSubFlowStep)
							tStepName = "Flow: " + tStepDefinition.TypeName;
						else
							tStepName = "Step: " + tStepDefinition.TypeName;

						if (!tComponents.Contains(tStepName))
							tComponents.Add(tStepName);
					}
				}

				MainWindow.Instance.listViewParts.ItemsSource = tComponents;
				MainWindow.Instance.listViewParts.Items.Filter = new Predicate<object>(Contains);
			}
			else
				MainWindow.Instance.listViewParts.Dispatcher.Invoke(new Action(In_Input));
		}

		private void textBoxComponentsFilter_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			if (e.Key == System.Windows.Input.Key.Return)
			{
				MainWindow.Instance.listViewParts.Items.Filter = new Predicate<object>(Contains);
			}
		}

		public void Initialize(object args)
		{
			//See: http://social.msdn.microsoft.com/Forums/en-US/wpf/thread/bfb49072-d3f3-4f2a-9f83-5a53ef8c5ee2/
			MainWindow.Instance.listViewParts.Items.SortDescriptions.Add(
				new System.ComponentModel.SortDescription());

			MainWindow.Instance.textBoxPartsFilter.KeyDown += textBoxComponentsFilter_KeyDown;
		}

		#region Private Helpers: Contains
		private bool Contains(object listBoxItem)
		{
			string tSearchString = listBoxItem.ToString().ToUpper();

			if (string.IsNullOrEmpty(MainWindow.Instance.textBoxPartsFilter.Text))
				return true;
			else
			{
				string[] tFilters = MainWindow.Instance.textBoxPartsFilter.Text.ToUpper().Split(' ');
				bool tContains = true;
				foreach (string tFilter in tFilters)
				{
					if (!tSearchString.Contains(tFilter))
					{
						tContains = false;
						break;
					}
				}

				return tContains;
			}
		}
		#endregion		
	}
}
