﻿using System.Collections.Generic;
using System.IO;
using AB3.Logging;

namespace AB3.DataTypes
{
	/// <summary>Contains the app definition with all created instances
	/// </summary>
	public class AppDefinition
	{
		/// <summary>Instance of ILogger to log the progress (LogLevel = 2)
		/// </summary>
		public ILogger ProgressLogger { get; set; }

		/// <summary>Instance of ILogger to log errors (All LogLevels)
		/// </summary>
		public ILogger ErrorLogger { get; set; }

		/// <summary>Instance of ILogger to log basic informations about the building (LogLevel = 1)
		/// </summary>
		public ILogger BasicLogger { get; set; }

		/// <summary>FlowDefinitions of the app defined in the FDL. Key is the name of the flow. For example "MainFlow"
		/// </summary>
		public Dictionary<string, FDLFlowDefinition> FlowDefinitions { get; set; }

		/// <summary>Exception because of parsing the FDL of flow definition
		/// </summary>
		public List<FDLException> ParseExceptions { get; set; }

		/// <summary>The text of the flow definition
		/// </summary>
		public List<string> FlowText { get; set; }

		/// <summary>Assemblies of the app to load
		/// </summary>
		public List<FileInfo> AssembliesToLoad { get; set; }

		/// <summary>Prepared instances of steps that the AppBuilder have to take instead of creating by activator.
		/// Keys are the full name of the step like MainFlow.StepInstance1
		/// </summary>
		public Dictionary<string, StepInstance> PreparedInstances { get; set; }

		/// <summary>All instance of steps of the flows. Keys are the full name of the step like MainFlow.StepInstance1
		/// </summary>
		public Dictionary<string, StepInstance> StepInstances { get; set; }

		/// <summary>The connections between the steps. 
		/// Keys are the full name of the connection like MainFlow.SubFlow1.StepInstance1.PinA->MainFlow.SubFlow2.StepInstance2.PinB
		/// </summary>
		public Dictionary<string, StepInstanceConnection> StepInstanceConnections { get; set; }

		/// <summary>The configure args to send to all steps which implements IConfigure
		/// </summary>
		public object ConfigureArg { get; set; }

		/// <summary>The initialize args to send to all steps which implements IInitialize
		/// </summary>
		public object InitializeArg { get; set; }

		/// <summary>Creates new App definition instance
		/// </summary>
		public AppDefinition()
		{
			this.ProgressLogger = new LogToNoWhere();
			this.BasicLogger = new LogToNoWhere();
			this.ErrorLogger = new LogToNoWhere();
			this.FlowDefinitions = new Dictionary<string, FDLFlowDefinition>();
			this.ParseExceptions = new List<FDLException>();
			this.FlowText = new List<string>();
			this.AssembliesToLoad = new List<FileInfo>();
			this.PreparedInstances = new Dictionary<string, StepInstance>();
			this.StepInstances = new Dictionary<string, StepInstance>();
			this.StepInstanceConnections = new Dictionary<string, StepInstanceConnection>();
		}
	}
}
