﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AB3.DataTypes
{
	/// <summary>DataType to store a single connection defined in FDL
	/// </summary>
	public class FDLWireDefinition
	{
		/// <summary>Line number where step was defined
		/// </summary>
		public int LineNumber { get; set; }

		/// <summary>Range of the remark
		/// </summary>
		public FDLTextRange Range { get; set; }

		/// <summary>The source of the connection
		/// </summary>
		public FDLConnector Source { get; set; }

		/// <summary>The target of the connection
		/// </summary>
		public FDLConnector Target { get; set; }

		/// <summary>Additional attribute of the wire
		/// </summary>
		public Dictionary<string, FDLAttribute> Attributes { get; set; }

		/// <summary>Creates a new instance to store a connection defined in FDL
		/// </summary>
		public FDLWireDefinition()
		{
			Range = new FDLTextRange();
			Source = new FDLConnector();
			Target = new FDLConnector();
			Attributes = new Dictionary<string, FDLAttribute>();
		}

		/// <summary>Returns the full name of the connection definition
		/// </summary>
		/// <returns>For example: StepA.PinB->StepC.PinD</returns>
		public override string ToString()
		{
			return string.Concat(Source.ToString(), "->", Target.ToString());
		}
	}
}
