﻿using System;
using System.IO;

namespace AB3.Logging
{
	/// <summary>Instance of ILogger to write message a simple text file
	/// </summary>
	public class LogToFile: ILogger
	{
		/// <summary>In case of multithreading use to synchronize the access
		/// </summary>
		private object _Lock = new object();

		/// <summary>the path and the filename of the text file
		/// </summary>
		private string _PathFileName = null;

		#region Ctor (with filename)
		/// <summary>Creates a new message logger to a simple text file
		/// </summary>
		/// <param name="pathFileName">path and filename of the logfile</param>
		public LogToFile(string pathFileName)
		{
			this._PathFileName = pathFileName;

			using (StreamWriter tLogWriter = new StreamWriter(this._PathFileName, false))
			{
				string tMessage = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss") + ": INFO: ProgressLogger startet!";
				tLogWriter.WriteLine(tMessage);
				tLogWriter.Close();
			}
		}
		#endregion

		/// <summary>Logs a message to a simple text file
		/// </summary>
		/// <param name="category">Category of the message</param>
		/// <param name="message">The message to log</param>
		public void Log(Category category, string message)
		{
			lock (_Lock)
			{
				using (StreamWriter tLogWriter = new StreamWriter(_PathFileName, true))
				{
					string tTimestamp = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
					string tMessageType;

					switch (category)
					{
						case Category.Info:
							tMessageType = "INFO";
							break;
						case Category.Warning:
							tMessageType = "WARNING";
							break;
						case Category.Error:
							tMessageType = "ERROR";
							break;
						default:
							tMessageType = "UNDEFINED MESSAGETYPE";
							break;
					}

					string tMessage = string.Format("{0}: {1}: {2}",
						tTimestamp, tMessageType, message);
					tLogWriter.WriteLine(tMessage);
					tLogWriter.Close();
				}
			}
		}
		
		/// <summary>Logs an info message
		/// </summary>
		/// <param name="message">Message content with placeholders of string.Format</param>
		/// <param name="args">arg to fill the placeholders of the message</param>
		public void LogInfo(string message, params string[] args)
		{
			if (args.Length == 0)
				this.Log(Category.Info, message);
			else
			{
				string tMessage = string.Format(message, args);
				this.Log(Category.Info, tMessage);
			}
		}

		/// <summary>Logs a warning message
		/// </summary>
		/// <param name="message">Message content with placeholders of string.Format</param>
		/// <param name="args">arg to fill the placeholders of the message</param>
		public void LogWarning(string message, params string[] args)
		{
			if (args.Length == 0)
				this.Log(Category.Warning, message);
			else
			{
				string tMessage = string.Format(message, args);
				this.Log(Category.Warning, tMessage);
			}
		}

		/// <summary>Logs an error message
		/// </summary>
		/// <param name="message">Message content with placeholders of string.Format</param>
		/// <param name="args">arg to fill the placeholders of the message</param>
		public void LogError(string message, params string[] args)
		{
			if (args.Length == 0)
				this.Log(Category.Error, message);
			else
			{
				string tMessage = string.Format(message, args);
				this.Log(Category.Error, tMessage);
			}
		}
	}
}
