﻿using System;

namespace AB3.Patterns
{
	/// <summary>Special Args for Stream
	/// </summary>
	public class ConvertStandardEventToStreamArgs
	{
		#region Properties: Sender, Args
		/// <summary>Sender of standard event
		/// </summary>
		public object Sender { get; private set; }

		/// <summary>Args of the standard event
		/// </summary>
		public EventArgs Args { get; private set; }
		#endregion

		#region Ctor (Sender, Args)
		internal ConvertStandardEventToStreamArgs(object sender, EventArgs args)
		{
			this.Sender = sender;
			this.Args = args;
		}
		#endregion

		/// <summary>Converts standard event to stream
		/// </summary>
		public class StandardEventToStream
		{
			/// <summary>Process pin for the standard event
			/// </summary>
			/// <param name="sender">Sender of the event</param>
			/// <param name="args">Content of the event</param>
			public void In_Input(object sender, EventArgs args)
			{
				this.Out_Output(new ConvertStandardEventToStreamArgs(sender, args));
			}

			/// <summary>Output pin to send
			/// </summary>
			public event Action<ConvertStandardEventToStreamArgs> Out_Output;
		}
	}
}
