﻿using System;
using System.IO;
using System.Reflection;
using AB3.DataTypes;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Loads the assemblies to the current app domain
	/// </summary>
	public class AssembliesLoader
	{
		/// <summary>Starts the loading of the assembliex
		/// </summary>
		/// <param name="app">The app definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start loading assemblies.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			Assembly[] tAssembliesOfDomain = AppDomain.CurrentDomain.GetAssemblies();
			int i = 0;

			foreach (FileInfo tFile in app.AssembliesToLoad)
			{
				bool tLoadAssembly = true;

				foreach (Assembly tAssemblyOfCurrentDomain in tAssembliesOfDomain)
				{
					if (tAssemblyOfCurrentDomain.Location == tFile.FullName)
						tLoadAssembly = false;
				}

				if (tLoadAssembly)
				{
					try
					{
						app.ProgressLogger.LogInfo("Load assembly to AppDomain: '{0}'", tFile.FullName);
						
						Assembly tComponent = Assembly.LoadFrom(tFile.FullName);
						i++;
					}
					catch (BadImageFormatException bex)
					{
						app.ErrorLogger.LogWarning("Couldn't load an assembly (BadImageFormatEception). Ignore this if it's no .NET lib. Exception.Message :");
						app.ErrorLogger.LogWarning(bex.Message);
						bex = null;
					}
					catch (FileLoadException ex)
					{
						app.ErrorLogger.LogError("Couldn't load the assemblies! (Maybe because of security)");
						app.ErrorLogger.LogError(ex.Message);
						app.ErrorLogger.LogError(@"Check this link for a possible solution: 'http://blog.amastaneh.com/2012/10/load-assembly-from-network-location.html'");
						throw ex;
					}
				}
				else
				{
					app.ProgressLogger.LogInfo("Assembly already part of AppDomain: '{0}'", tFile.FullName);
				}
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished loading assemblies in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
