﻿using System;
using AB3.DataTypes;
using AB3.Helpers;

namespace AB3.Steps
{
	/// <summary>Find the flow definitions in FDL
	/// </summary>
	public class FDLFlowDefinitionsParser
	{
		/// <summary>Starts finding the FDL in a giver app definition
		/// </summary>
		/// <param name="app">App definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start parsing flow definitions in '{0}' lines.", app.FlowText.Count.ToString());
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			for (int i = 0; i < app.FlowText.Count; i++)
			{
				if (app.FlowText[i].StartsWith("FlowDefinition:"))
				{
					FDLFlowDefinition tFlowDefinition = new FDLFlowDefinition();
					tFlowDefinition.Name = LineConverter.ToFlowDefinitionName(app.FlowText[i]);
					if (app.FlowDefinitions.ContainsKey(tFlowDefinition.Name))
					{
						FDLException tEx = new FDLException("Flow definition with duplicate name found!",
							i + 1, app.FlowText[i]);
						app.ErrorLogger.LogError(tEx.ToString());
					}
					else
					{
						app.FlowDefinitions.Add(tFlowDefinition.Name, tFlowDefinition);
						app.ProgressLogger.LogInfo("Flow Definition with name '{0}' found in line {1}",
							tFlowDefinition.Name, (i + 1).ToString());
					}

					if (app.FlowText[i].EndsWith("{"))
						tFlowDefinition.Range.StartLine = i;
					else
						tFlowDefinition.Range.StartLine = StringEx.FindContentStart(app.FlowText, i + 1);

					if (tFlowDefinition.Range.StartLine != -1)
					{
						tFlowDefinition.Range.EndLine = StringEx.FindContentEnd(app.FlowText, tFlowDefinition.Range.StartLine);

						if (tFlowDefinition.Range.EndLine != -1)
							i = tFlowDefinition.Range.EndLine;
						else
						{
							FDLException tEx = new FDLException("End of flow definition is missing",
								i + 1, app.FlowText[i]);
							app.ParseExceptions.Add(tEx);
							app.ErrorLogger.LogError(tEx.ToString());
						}
					}
					else
					{
						FDLException tEx = new FDLException("Content of flow definition is missing",
							i + 1, app.FlowText[i]);
						app.ParseExceptions.Add(tEx);
						app.ErrorLogger.LogError(tEx.ToString());
					}					
				}
				else if (app.FlowText[i] == string.Empty) // Ignore line because of empty
				{ }
				else if (app.FlowText[i].StartsWith("//")) // Ignow line becaus it's a comment
				{ }
				else
				{
					FDLException tEx = new FDLException("Unknown keyword found!",
						i + 1, app.FlowText[i]);
					app.ParseExceptions.Add(tEx);
					app.ErrorLogger.LogError(tEx.ToString());
				}
			}

			#region Logging
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished parsing {0} flow definitions in {1} ms.",
				app.FlowDefinitions.Count.ToString(), tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
