﻿using System;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Finds the steps of the flows defined in the FDL
	/// </summary>
	public class FDLStepDefinitionsParser
	{
		/// <summary>Starts finding the steps
		/// </summary>
		/// <param name="app">The app definition to work with</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding steps.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			foreach (FDLFlowDefinition tFlowDefinition in app.FlowDefinitions.Values)
			{
				app.ProgressLogger.LogInfo("Start finding steps of flow '{0}'", tFlowDefinition.Name);

				for (int i = tFlowDefinition.Range.StartLine; i <= tFlowDefinition.Range.EndLine; i++)
				{
					if ((app.FlowText[i].StartsWith("Step:")) || (app.FlowText[i].StartsWith("Flow:")))
					{
						FDLStepDefinition tStepDefinition = LineConverter.ToStepDefinition(app, i);
						if (tFlowDefinition.Steps.Collection.ContainsKey(tStepDefinition.CustomName))
						{
							FDLException tEx;
							if (tStepDefinition.IsSubFlowStep)
								tEx = new FDLException("SubFlow with duplicate name found!", i + 1, app.FlowText[i]);
							else
								tEx = new FDLException("Step with duplicate name found!", i + 1, app.FlowText[i]);
							app.ParseExceptions.Add(tEx);
							app.ProgressLogger.LogError(tEx.ToString());
						}
						else
						{
							tFlowDefinition.Steps.Collection.Add(tStepDefinition.CustomName, tStepDefinition);
							if (tStepDefinition.IsSubFlowStep)
								app.ProgressLogger.LogInfo("SubFlow with name '{0}' found in line {1}",
									tStepDefinition.CustomName, (i + 1).ToString());
							else
								app.ProgressLogger.LogInfo("Step with name '{0}' found in line {1}",
									tStepDefinition.CustomName, (i + 1).ToString());
						}

						if (app.FlowText[i].EndsWith("{"))
							tStepDefinition.Range.StartLine = i;
						else
							tStepDefinition.Range.StartLine = StringEx.FindContentStart(app.FlowText, i + 1);

						if (tStepDefinition.Range.StartLine != -1)
						{
							tStepDefinition.Range.EndLine = StringEx.FindContentEnd(app.FlowText, tStepDefinition.Range.StartLine);

							if (tStepDefinition.Range.EndLine != -1)
							{
								i = tStepDefinition.Range.EndLine;
							}
							else
							{
								FDLException tEx = new FDLException("End of step definition is missing",
								i + 1, app.FlowText[i]);
								app.ParseExceptions.Add(tEx);
								app.ProgressLogger.LogError(tEx.ToString());
							}
						}
						else // No content defined => OK
						{
							tStepDefinition.Range.StartLine = i;
							tStepDefinition.Range.EndLine = i;
						}
					}

				}	
			
				app.ProgressLogger.LogInfo("{0} steps found.", tFlowDefinition.Steps.Collection.Count.ToString());
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished finding steps in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}		

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
