﻿using System;
using System.Text.RegularExpressions;
using AB3.DataTypes;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Replaces tabs and multiple spaces in the given flow FDL
	/// </summary>
	public class FDLTabReplacer
	{
		/// <summary>Starts the replacing of tabs and multiple spaces
		/// </summary>
		/// <param name="app">The app definition to modify</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start replacing tabs and multiple spaces of '{0}' lines.", app.FlowText.Count.ToString());
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			string tString;
			for (int i = 0; i < app.FlowText.Count; i++)
			{				
				tString = Regex.Replace(app.FlowText[i], "\t", " ").Trim(); // Tab by space
				tString = Regex.Replace(tString, " {2,}", " "); // Doublespace by space
				app.FlowText[i] = tString;
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished replacing tabs and multiple spaces in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
