﻿using System;
using AB3.DataTypes;
using AB3.Helpers;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Finds connections of steps defined in FDL
	/// </summary>
	public class FDLWireDefinitionsParser
	{
		/// <summary>Starts the finding of connections
		/// </summary>
		/// <param name="app">The app definition</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start finding wires.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			foreach (FDLFlowDefinition tFlowDefinition in app.FlowDefinitions.Values)
			{
				app.ProgressLogger.LogInfo("Start finding connections of flow '{0}'", tFlowDefinition.Name);

				for (int i = tFlowDefinition.Range.StartLine; i <= tFlowDefinition.Range.EndLine; i++)
				{
					if (app.FlowText[i].StartsWith("Wire:"))
					{
						FDLWireDefinition tWireDefinition = LineConverter.ToWireDefinition(app, i);
						tFlowDefinition.Wires.Collection.Add(tWireDefinition);
						app.ProgressLogger.LogInfo("Wire '{0}' found.", tWireDefinition.ToString());

						if (app.FlowText[i].EndsWith("{"))
							tWireDefinition.Range.StartLine = i;
						else
							tWireDefinition.Range.StartLine = StringEx.FindContentStart(app.FlowText, i + 1);

						if (tWireDefinition.Range.StartLine != -1)
						{
							tWireDefinition.Range.EndLine = StringEx.FindContentEnd(app.FlowText, tWireDefinition.Range.StartLine);
							if (tWireDefinition.Range.EndLine != -1)
							{
								i = tWireDefinition.Range.EndLine;
							}
							else
							{
								FDLException tEx = new FDLException("End of wire definition is missing",
									i + 1, app.FlowText[i]);
								app.ParseExceptions.Add(tEx);
								app.ProgressLogger.LogError(tEx.ToString());
							}
						}
						else // No content defined => OK
						{
							tWireDefinition.Range.StartLine = i;
							tWireDefinition.Range.EndLine = i;
						}
					}
				}
			
				app.ProgressLogger.LogInfo("{0} connections found.", tFlowDefinition.Wires.Collection.Count.ToString());
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished finding wires in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
