﻿using System;
using AB3.DataTypes;
using AB3.Interfaces;
using AB3.Logging;

namespace AB3.Steps
{
	/// <summary>Initializes all steps that implement IInitialize
	/// </summary>
	public class StepInstancesInitializer
	{
		/// <summary>Start the initialization
		/// </summary>
		/// <param name="app">The app definition to work with</param>
		public void In_Input(AppDefinition app)
		{
			#region Logging & Profiling
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			app.ProgressLogger.LogInfo("Start initializing steps.");
			app.ProgressLogger.LogInfo("---------------------------------------------------");
			DateTime tStart = DateTime.Now;
			#endregion

			IInitialize tInitializeInstance;
			foreach (StepInstance tStepInstance in app.StepInstances.Values)
			{
				if (tStepInstance.StepObject != null)
				{
					tInitializeInstance = tStepInstance.StepObject as IInitialize;
					if (tInitializeInstance != null)
					{
						app.ProgressLogger.LogInfo("Initialize step '{0}'", tStepInstance.StepType.ToString());
						tInitializeInstance.Initialize(app.InitializeArg);
					}
				}
			}

			#region Logging & Profiling
			TimeSpan tDuration = DateTime.Now - tStart;
			app.BasicLogger.LogInfo("Finished initializing steps in {0} ms.",
				tDuration.Milliseconds.ToString());
			#endregion

			Out_Output(app);
		}

		/// <summary>Sends the app definition to the next step
		/// </summary>
		public event Action<AppDefinition> Out_Output;
	}
}
