﻿using System;
using System.Timers;

namespace AB3.AppDesigner.BaseClasses
{
	public abstract class Scheduler<T> where T: class
	{
		private double _WaitTime;
		private DateTime _ModDat;
		private T _ChangedContent;
		private Timer _TimeToCheckTimer;

		/// <summary>Slows down a flow because massive changings. Wait until no new changes arrives for specific time.
		/// 
		/// </summary>
		/// <param name="waitTime">Time (in seconds) to wait after last changing was received.</param>
		public Scheduler(double waitTime)
		{
			_WaitTime = waitTime;
			_ModDat = DateTime.Now;
			_ChangedContent = null;
			_TimeToCheckTimer = new Timer(waitTime / 4);
			_TimeToCheckTimer.Elapsed += TimeToCheckTimer_Elapsed;
		}

		private void TimeToCheckTimer_Elapsed(object sender, ElapsedEventArgs e)
		{
			double tElapsedSeconds = DateTime.Now.Subtract(this._ModDat).TotalSeconds;
			if ((tElapsedSeconds > _WaitTime) && (_ChangedContent != null))
			{
				this._TimeToCheckTimer.Stop();
				Out_Output(_ChangedContent);
			}
		}

		public void In_Input(T content)
		{
			this._ModDat = DateTime.Now;
			this._ChangedContent = content;

			if (!this._TimeToCheckTimer.Enabled)
				this._TimeToCheckTimer.Start();
		}

		public event Action<T> Out_Output;
	}
}
