﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using AB3.DataTypes;

namespace AB3.AppDesigner.Converters
{
	public static class FDLAttributeConverter
	{
		public static double ToDouble(Dictionary<string, FDLAttribute> attributes, string name, double defaultValue)
		{
			if (attributes.ContainsKey(name))
				return ToDouble(attributes[name], defaultValue);
			else
				return defaultValue;
		}

		public static double ToDouble(FDLAttribute attribute, double defaultValue)
		{
			try
			{
				return Convert.ToDouble(attribute.Value);
			}
			catch
			{
				return defaultValue;
			}
		}

		public static Thickness ToPosition(Dictionary<string, FDLAttribute> attributes, double defaultLeft, double defaultTop)
		{
			Thickness tReturn = new Thickness();
			if (attributes.ContainsKey("Position"))
			{
				string tPosition = attributes["Position"].Value;
				string[] tParts = tPosition.Split(new string[] { "," }, StringSplitOptions.None);
				tReturn.Left = ToDouble(tParts[0], defaultLeft);
				if (tParts.Length != 1)
					tReturn.Top = ToDouble(tParts[1], defaultTop);
				else
					tReturn.Top = defaultTop;
			}
			else
			{
				tReturn.Left = defaultLeft;
				tReturn.Top = defaultTop;
			}

			return tReturn;
		}

		public static Thickness ToPointPosition(FDLAttribute attribute, double defaultLeft, double defaultTop, double offset)
		{
			Thickness tReturn = new Thickness();
			string[] tParts = attribute.Value.Split(new string[] { "," }, StringSplitOptions.None);
			tReturn.Left = ToDouble(tParts[0], defaultLeft) + offset;
			if (tParts.Length != 1)
				tReturn.Top = ToDouble(tParts[1], defaultTop) + offset;
			else
				tReturn.Top = defaultTop;

			return tReturn;
		}

		public static void SetSize(Dictionary<string, FDLAttribute> attributes, Border control, double defaultWidth, double defaultHeight)
		{
			if (attributes.ContainsKey("Size"))
			{
				string tSize = attributes["Size"].Value;
				string[] tParts = tSize.Split(new string[] { "," }, StringSplitOptions.None);
				control.Width = ToDouble(tParts[0], defaultWidth);
				if (tParts.Length != 1)
					control.Height = ToDouble(tParts[1], defaultHeight);
				else
					control.Height = defaultHeight;
			}
			else
			{
				control.Width = defaultWidth;
				control.Height = defaultHeight;
			}
		}

		public static double ToDouble(string valueString, double defaultValue)
		{
			try
			{
				return Convert.ToDouble(valueString);
			}
			catch
			{
				return defaultValue;
			}
		}

		public static string ToString(FDLAttribute attribute)
		{
			return string.Concat(attribute.Name, " = ", '"', attribute.Value, '"');
		}

		public static string ToString(IEnumerable<FDLAttribute> attributes)
		{
			string tReturn = string.Empty;

			foreach (FDLAttribute tAttribute in attributes)
			{
				tReturn += ToString(tAttribute) + " ";
			}

			return tReturn;
		}
	}
}
