﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace AB3.AppDesigner.DataTypes
{
	public class AppConfig: INotifyPropertyChanged
	{
		public static AppConfig Instance = null;

		private Dictionary<string, string> _ConfigValues;

		#region ctor
		public AppConfig(Dictionary<string, string> configValues)
		{
			Instance = this;
			this._ConfigValues = configValues;
		}
		#endregion

		#region GetValue As String, Boolean, Int, Double
		public string GetValueAsString(string parameter)
		{
			if (this._ConfigValues.ContainsKey(parameter))
				return this._ConfigValues[parameter];
			else
				throw new ArgumentOutOfRangeException("name", "AppConfig: Unknown config name '" + parameter + "'");
		}

		public bool GetValueAsBoolean(string parameter)
		{
			return bool.Parse(this.GetValueAsString(parameter));
		}

		public int GetValueToInt(string parameter)
		{
			return int.Parse(GetValueAsString(parameter));
		}

		public double GetValueAsDouble(string parameter)
		{
			return (double.Parse(GetValueAsString(parameter)) / 100);
		}
		#endregion

		#region SetValue for String, Boolean, Int, Double
		public void SetValue(string parameter, string value)
		{
			if (this._ConfigValues.ContainsKey(parameter))
			{
				string tOldValue = this._ConfigValues[parameter];
				if (tOldValue != value)
				{
					this._ConfigValues[parameter] = value;
					NotifyPropertyChanged(parameter);
				}
			}
			else
				throw new ArgumentOutOfRangeException("name", "AppConfig: Unknown config name '" + parameter + "'");
		}

		public void SetValue(string parameter, bool value)
		{
			SetValue(parameter, value.ToString());
		}

		public void SetValue(string parameter, int value)
		{
			SetValue(parameter, value.ToString());
		}

		public void SetValue(string parameter, double value)
		{
			SetValue(parameter, (value * 100).ToString("F0"));
		}
		#endregion

		#region INotifyPropertyChanged
		private void NotifyPropertyChanged(String info)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(info));
			}
		}
		public event PropertyChangedEventHandler PropertyChanged;
		#endregion
	}
}
