﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Media.Effects;
using AB3.AppDesigner.Adorners;
using AB3.AppDesigner.Converters;
using AB3.DataTypes;
using AB3.Helpers;

namespace AB3.AppDesigner.Decorators
{
	public class StepDecorator
	{
		#region Static Layout definitions
		private static SolidColorBrush _SingleStepBackgroundBrush;
		private static SolidColorBrush _FlowStepBackgroundBrush;
		private static SolidColorBrush _BorderBrush;
		private static Thickness _SingleStepBorderThickness;
		private static Thickness _FlowStepBorderThickness;
		private static DropShadowEffect _ShadowEffect;
		private static CornerRadius _CornerRadius;
		#endregion

		public FDLStepDefinition Definition { get; set; }
		public Border View { get; private set; }
		private BorderAdorner _BorderAdorner = null;

		#region Static ctor
		static StepDecorator()
		{
			_FlowStepBackgroundBrush = new SolidColorBrush(Colors.Gold);
			_SingleStepBackgroundBrush = new SolidColorBrush(Colors.LightBlue);
			_BorderBrush = new SolidColorBrush(Colors.Black);
			_SingleStepBorderThickness = new Thickness(1);
			_FlowStepBorderThickness = new Thickness(2);
			_CornerRadius = new CornerRadius(4);

			//See: http://msdn.microsoft.com/de-de/library/vstudio/ms748273.aspx
			_ShadowEffect = new DropShadowEffect();
			_ShadowEffect.ShadowDepth = 4;
			_ShadowEffect.Color = Colors.Black;
			_ShadowEffect.Opacity = 0.5;
			_ShadowEffect.BlurRadius = 4;
		}
		#endregion

		#region Instance ctor
		public StepDecorator(FDLStepDefinition stepDefinition)
		{
			this.Definition = stepDefinition;

			#region TextBlock
			TextBlock tTextBlock = new TextBlock();
			if (stepDefinition.TypeName != stepDefinition.CustomName)
				tTextBlock.Text = StringEx.SplitCamelCase(string.Concat(stepDefinition.TypeName, " => ", stepDefinition.CustomName));
			else
				tTextBlock.Text = StringEx.SplitCamelCase(stepDefinition.TypeName);
			tTextBlock.TextWrapping = TextWrapping.Wrap;
			tTextBlock.HorizontalAlignment = HorizontalAlignment.Center;
			tTextBlock.VerticalAlignment = VerticalAlignment.Center;
			#endregion

			this.View = new Border();
			if (stepDefinition.IsSubFlowStep)
			{
				this.View.Background = _FlowStepBackgroundBrush;
				this.View.BorderThickness = _FlowStepBorderThickness;
			}
			else
			{
				this.View.Background = _SingleStepBackgroundBrush;
				this.View.BorderThickness = _SingleStepBorderThickness;
			}
			this.View.Child = tTextBlock;
			this.View.BorderBrush = _BorderBrush;
			this.View.CornerRadius = _CornerRadius;
			this.View.Margin = FDLAttributeConverter.ToPosition(stepDefinition.Attributes, 10, 10);
			this.View.HorizontalAlignment = HorizontalAlignment.Left;
			this.View.VerticalAlignment = VerticalAlignment.Top;
			FDLAttributeConverter.SetSize(stepDefinition.Attributes, this.View, 100, 50);
			this.View.Effect = _ShadowEffect;
			this.View.MouseDown += View_MouseDown;
		}
		#endregion

		private void View_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (e.ClickCount == 1)
			{
				AdornerLayer tAdornerLayer = AdornerLayer.GetAdornerLayer(this.View);

				if ((tAdornerLayer != null) && (_BorderAdorner != null))
				{
					tAdornerLayer.Remove(_BorderAdorner);
					_BorderAdorner = null;
				}
				else if (_BorderAdorner == null)
				{
					_BorderAdorner = new BorderAdorner(this.View);
					_BorderAdorner.BorderChanged += _BorderAdorner_BorderChanged;
					tAdornerLayer.Add(_BorderAdorner);
				}

				e.Handled = true;
			}
			else if (e.ClickCount == 2)
			{
				Out_ViewDoubleClicked(this.Definition);
				e.Handled = true;
			}
		}

		private void _BorderAdorner_BorderChanged(Border obj)
		{
			Out_ViewChanged(this.Definition, this.View);
		}

		public event Action<FDLStepDefinition, Border> Out_ViewChanged;
		public event Action<FDLStepDefinition> Out_ViewDoubleClicked;
	}
}
