﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireCaptionDecorator
	{
		#region Static Layout definitions
		private static Thickness _CaptionTextBlockPadding;
		#endregion

		#region Instance properties and fields
		public TextBlock View { get; private set; }
		private Line _Line;
		private FDLWireDefinition _WireDefinition;
		#endregion

		#region Static ctor
		static WireCaptionDecorator()
		{
			_CaptionTextBlockPadding = new Thickness(2);
		}
		#endregion

		#region Instance ctor
		public WireCaptionDecorator(FDLWireDefinition wireDefinition, int orderNumber, Line line)
		{
			_Line = line;
			_WireDefinition = wireDefinition;
			this.View = new TextBlock();
			this.View.Padding = _CaptionTextBlockPadding;
			if (wireDefinition.Attributes.ContainsKey("Caption"))
				this.View.Text = string.Concat(orderNumber.ToString(), ". ", wireDefinition.Attributes["Caption"].Value);
			else
				this.View.Text = string.Concat(orderNumber.ToString(), ".");
			this.View.Background = System.Windows.Media.Brushes.LightYellow;
			this.View.Opacity = 0.9d;
			this.View.TextWrapping = TextWrapping.NoWrap;
			this.View.HorizontalAlignment = HorizontalAlignment.Left;
			this.View.VerticalAlignment = VerticalAlignment.Top;			
			this.View.Visibility = Visibility.Hidden;
			this.View.MouseDown += View_MouseDown;
		}

		#endregion

		#region View-EventHandlers: MouseDown
		private void View_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (e.ClickCount == 1)
			{
				ViewSingleClicked();
				e.Handled = true;
			}
			else if (e.ClickCount == 2)
			{
				ViewDoubleClicked(_WireDefinition);
				e.Handled = true;
			}
		}
		#endregion

		#region Public Methods: RefreshView
		public void RefreshView()
		{
			Thickness tMargin = new Thickness();
			tMargin.Left = (_Line.X1 + _Line.X2) / 2;
			tMargin.Top = (_Line.Y1 + _Line.Y2) / 2;
			this.View.Margin = tMargin;

			this.View.UpdateLayout();
			tMargin = new Thickness();
			tMargin.Left = this.View.Margin.Left - this.View.RenderSize.Width / 2;
			tMargin.Top = this.View.Margin.Top - this.View.RenderSize.Height / 2;
			this.View.Margin = tMargin;
			this.View.Visibility = Visibility.Visible;
		}
		#endregion

		#region Public Events: ViewSingleClicked, ViewDoubleClicked
		public event Action ViewSingleClicked;
		public event Action<FDLWireDefinition> ViewDoubleClicked;
		#endregion
	}
}
