﻿using System;
using System.Windows;
using System.Windows.Controls;
using AB3.AppDesigner.BaseClasses;
using AB3.AppDesigner.Helpers;
using AB3.DataTypes;

namespace AB3.AppDesigner.Decorators
{
	public class WireLinePointToPointDecorator : WireLineDecoratorBase
	{
		WirePointDecorator _SourcePointDecorator;
		WirePointDecorator _TargetPointDecorator;
		private Point _NewSourcePointPosition;
		private Point _NewTargetPointPosition;

		#region Ctor
		public WireLinePointToPointDecorator(FDLWireDefinition wireDefinition, WirePointDecorator sourcePointDecorator, WirePointDecorator targetPointDecorator): base(wireDefinition)
		{
			_SourcePointDecorator = sourcePointDecorator;
			_TargetPointDecorator = targetPointDecorator;
			_NewSourcePointPosition = new Point();
			_NewTargetPointPosition = new Point();

			Border tSource = sourcePointDecorator.View;
			Border tTarget = targetPointDecorator.View;

			base.View.X1 = tSource.Margin.Left + tSource.Width / 2;
			base.View.Y1 = tSource.Margin.Top + tSource.Height / 2;
			base.View.X2 = tTarget.Margin.Left + tTarget.Width / 2;
			base.View.Y2 = tTarget.Margin.Top + tTarget.Height / 2;
		}
		#endregion

		#region Overrides: SelectWireLineDecorator
		public override void SelectWireLineDecorator()
		{
			base.SelectWireLineDecorator();

			MenuItem tStartThumbMenuItem = new MenuItem();
			tStartThumbMenuItem.Header = string.Format("Remove this point attribute ({0})", _SourcePointDecorator.Name);
			tStartThumbMenuItem.Click += (a, b) => RemovePointAttribute(base.Definition, _SourcePointDecorator.Name);

			ContextMenu tStartThumbContextMenu = new ContextMenu();
			tStartThumbContextMenu.Items.Add(tStartThumbMenuItem);
			WireLineDecoratorBase._LineAdorner._StartThumb.ContextMenu = tStartThumbContextMenu;

			MenuItem tTargetThumbMenuItem = new MenuItem();
			tTargetThumbMenuItem.Header = string.Format("Remove this point attribute ({0})", _TargetPointDecorator.Name);
			tTargetThumbMenuItem.Click += (a, b) => RemovePointAttribute(base.Definition, _TargetPointDecorator.Name);

			ContextMenu tTargetThumbContextMenu = new ContextMenu();
			tTargetThumbContextMenu.Items.Add(tTargetThumbMenuItem);
			WireLineDecoratorBase._LineAdorner._TargetThumb.ContextMenu = tTargetThumbContextMenu;
		}
		#endregion

		#region Overrides: LineAdorner_StartThumbChanged, _TargetThumbChanged, _ChangeFinished, _AddNewPointAttribute
		protected override void LineAdorner_StartThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			_NewSourcePointPosition = ChartHelper.GetSnapPoint(position);
			line.X1 = _NewSourcePointPosition.X;
			line.Y1 = _NewSourcePointPosition.Y;
			RefreshViewOfPreviousDecorators(this, _NewSourcePointPosition);
		}

		protected override void LineAdorner_TargetThumbChanged(System.Windows.Shapes.Line line, Point position)
		{
			_NewTargetPointPosition = ChartHelper.GetSnapPoint(position);
			line.X2 = _NewTargetPointPosition.X;
			line.Y2 = _NewTargetPointPosition.Y;
			RefreshViewOfNextDecorators(this, _NewTargetPointPosition);
		}

		protected override void LineAdorner_ChangeFinished()
		{
			if ((_NewSourcePointPosition.X != 0) || (_NewSourcePointPosition.Y != 0))
			{
				SourcePointChanged(base.Definition, _SourcePointDecorator.Name, _NewSourcePointPosition);
			}
			else if ((_NewTargetPointPosition.X != 0) || (_NewTargetPointPosition.Y != 0))
			{
				TargetPointChanged(base.Definition, _TargetPointDecorator.Name, _NewTargetPointPosition);
			}
		}

		protected override void LineAdorner_AddNewPointAttribute(Point position)
		{
			AddNewPointAttribute(base.Definition, _TargetPointDecorator.Name, position);
		}
		#endregion

		public event Action<WireLineDecoratorBase, Point> RefreshViewOfNextDecorators;
		public event Action<WireLineDecoratorBase, Point> RefreshViewOfPreviousDecorators;
		public event Action<FDLWireDefinition, string, Point> SourcePointChanged;
		public event Action<FDLWireDefinition, string, Point> TargetPointChanged;
		public event Action<FDLWireDefinition, string, Point> AddNewPointAttribute;
		public event Action<FDLWireDefinition, string> RemovePointAttribute;
	}
}
