﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;

namespace AB3.AppDesigner.EditWindows
{
	/// <summary>
	/// Interaction logic for EditWireWindow.xaml
	/// </summary>
	public partial class EditWireWindow : Window
	{
		#region Fields & Properties: EditWireArgument
		private EditWireArgument _EditWireArgument;
		public FDLConnector WireSource { get; private set; }
		public FDLConnector WireTarget { get; private set; }
		#endregion

		#region Private Helpers: ChangeInterfaceViews, ChangeFlowStepViews
		private void ChangeInterfaceViews(EditWireArgument argument)
		{
			if (argument.WireSources.Interfaces.Count == 0)
			{
				this.tabItemSourceInterfaces.Visibility = System.Windows.Visibility.Collapsed;
				this.tabItemSourceFlows.IsSelected = true;
			}
			else
				this.listBoxSourceInterfaces.ItemsSource = argument.WireSources.Interfaces;

			if (argument.WireTargets.Interfaces.Count == 0)
			{
				this.tabItemTargetInterfaces.Visibility = System.Windows.Visibility.Collapsed;
				this.tabItemTargetFlows.IsSelected = true;
			}
			else
				this.listBoxTargetInterfaces.ItemsSource = argument.WireTargets.Interfaces;
		}

		private void ChangeFlowStepViews(EditWireArgument argument)
		{
			if (argument.WireSources.FlowSteps.Count == 0)
			{
				if (this.tabItemSourceFlows.IsSelected)
					this.tabItemSourceSteps.IsSelected = true;
				this.tabItemSourceFlows.Visibility = System.Windows.Visibility.Collapsed;
			}
			else
				this.listBoxSourceFlowSteps.ItemsSource = argument.WireSources.FlowSteps.Keys.ToList();

			if (argument.WireTargets.FlowSteps.Count == 0)
			{
				if (this.tabItemTargetFlows.IsSelected)
					this.tabItemTargetSteps.IsSelected = true;
				this.tabItemTargetFlows.Visibility = System.Windows.Visibility.Collapsed;
			}
			else
				this.listBoxTargetFlowSteps.ItemsSource = argument.WireTargets.FlowSteps.Keys.ToList();
		}

		private void ChangeSingleStepViews(EditWireArgument argument)
		{
			if (argument.WireSources.SingleSteps.Count == 0)
				this.tabItemSourceSteps.Visibility = System.Windows.Visibility.Collapsed;
			else
				this.listBoxSourceSingleSteps.ItemsSource = argument.WireSources.SingleSteps.Keys.ToList();

			if (argument.WireTargets.SingleSteps.Count == 0)
				this.tabItemTargetSteps.Visibility = System.Windows.Visibility.Collapsed;
			else
				this.listBoxTargetSingleSteps.ItemsSource = argument.WireTargets.SingleSteps.Keys.ToList();
		}
		#endregion

		#region ctors
		public EditWireWindow(FDLConnector wireSource, FDLConnector wireTarget, EditWireArgument argument): this()
		{
			this.WireSource = new FDLConnector();
			this.WireSource.Step = wireSource.Step;
			this.WireSource.Pin = wireSource.Pin;
			this.WireTarget = new FDLConnector();
			this.WireTarget.Step = wireTarget.Step;
			this.WireTarget.Pin = wireTarget.Pin;
			this.RefreshView();
			_EditWireArgument = argument;
			ChangeInterfaceViews(_EditWireArgument);
			ChangeFlowStepViews(_EditWireArgument);
			ChangeSingleStepViews(_EditWireArgument);
		}

		private void RefreshView()
		{
			this.textBoxDefinition.Text = string.Concat(this.WireSource.Step, ".", this.WireSource.Pin, " -> ", this.WireTarget.Step, ".", this.WireTarget.Pin);
		}

		public EditWireWindow()
		{
			InitializeComponent();
			this.textBoxDefinition.LostFocus += textBoxDefinition_LostFocus;
			this.listBoxSourceInterfaces.MouseDoubleClick += listBoxSourceInterfaces_MouseDoubleClick;
			this.listBoxTargetInterfaces.MouseDoubleClick += listBoxTargetInterfaces_MouseDoubleClick;
			this.listBoxSourceFlowSteps.SelectionChanged += listBoxSourceFlowSteps_SelectionChanged;
			this.listBoxTargetFlowSteps.SelectionChanged += listBoxTargetFlowSteps_SelectionChanged;
			this.listBoxSourceSingleSteps.SelectionChanged += listBoxSourceSteps_SelectionChanged;
			this.listBoxSourceFlowSteps.MouseDoubleClick += listBoxSourceFlowSteps_MouseDoubleClick;
			this.listBoxTargetSingleSteps.SelectionChanged += listBoxTargetSingleSteps_SelectionChanged;
			this.listBoxTargetFlowSteps.MouseDoubleClick += listBoxTargetFlowSteps_MouseDoubleClick;
			this.listBoxSourceFlowStepInterfaces.MouseDoubleClick += listBoxSourceFlowStepInterfaces_MouseDoubleClick;
			this.listBoxTargetFlowStepInterfaces.MouseDoubleClick += listBoxTargetFlowStepInterfaces_MouseDoubleClick;
			this.listBoxSourceSingleSteps.MouseDoubleClick += listBoxSourceSteps_MouseDoubleClick;
			this.listBoxSourceSingleStepPins.MouseDoubleClick += listBoxSourceStepPins_MouseDoubleClick;
			this.listBoxTargetSingleSteps.MouseDoubleClick += listBoxTargetSingleSteps_MouseDoubleClick;
			this.listBoxTargetSingleStepPins.MouseDoubleClick += listBoxTargetSingleStepPins_MouseDoubleClick;
		}		
		#endregion

		#region Definition-TextBox-EventHanlder: LostFocus
		private void textBoxDefinition_LostFocus(object sender, RoutedEventArgs e)
		{
			string tText = string.Concat(this.WireSource.Step, ".", this.WireSource.Pin, " -> ", this.WireTarget.Step, ".", this.WireTarget.Pin);
			if (this.textBoxDefinition.Text != tText) // Need to parse input
			{
				string[] tLineParts = this.textBoxDefinition.Text.Trim().Split(new string[] { "->" }, StringSplitOptions.None);

				if (tLineParts.Length == 2)
				{
					this.WireSource = GetConnectorFromString(tLineParts[0]);
					this.WireTarget = GetConnectorFromString(tLineParts[1]);
				}
			}
		}

		private FDLConnector GetConnectorFromString(string text)
		{
			string[] tParts = text.Split(new string[] { "." }, StringSplitOptions.None);
			FDLConnector tReturn = new FDLConnector();

			if (tParts.Length > 1)
			{
				tReturn.Pin = tParts[tParts.Length - 1].Trim();
				tReturn.Step = tParts[tParts.Length - 2].Trim();
			}
			else
			{
				tReturn.Step = tParts[0].Trim();
				tReturn.Pin = string.Empty;
			}

			return tReturn;
		}
		#endregion

		#region ButtonEvent-Handler: OKClicked, CancelClicked
		private void OKClicked(object sender, ExecutedRoutedEventArgs e)
		{
			this.DialogResult = true;
			this.Close();
		}

		private void CancelClicked(object sender, ExecutedRoutedEventArgs e)
		{
			this.DialogResult = false;
			this.Close();
		}
		#endregion

		#region InterfaceViews Event-Handler
		private void listBoxSourceInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxSourceInterfaces.SelectedItem != null)
			{
				this.WireSource.Step = "In";
				this.WireSource.Pin = this.listBoxSourceInterfaces.SelectedItem.ToString();
			}
			else
			{
				this.WireSource.Step = string.Empty;
				this.WireSource.Pin = string.Empty;				
			}
			this.RefreshView();
		}

		private void listBoxTargetInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxTargetInterfaces.SelectedItem != null)
			{
				this.WireTarget.Step = "Out";
				this.WireTarget.Pin = this.listBoxTargetInterfaces.SelectedItem.ToString();
			}
			else
			{
				this.WireTarget.Step = string.Empty;
				this.WireTarget.Pin = string.Empty;
			}
			this.RefreshView();
		}
		#endregion

		#region FlowStepViews Event-Handler
		private void listBoxSourceFlowSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (this.listBoxSourceFlowSteps.SelectedItem != null)
			{
				string tKey = this.listBoxSourceFlowSteps.SelectedItem.ToString();
				this.listBoxSourceFlowStepInterfaces.ItemsSource =_EditWireArgument.WireSources.FlowSteps[tKey];
			}
			else
			{
				this.listBoxSourceFlowStepInterfaces.ItemsSource = null;
				this.listBoxSourceFlowStepInterfaces.Items.Clear();
			}
		}

		private void listBoxSourceFlowSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxSourceFlowSteps.SelectedItem != null)
				this.WireSource.Step = this.listBoxSourceFlowSteps.SelectedItem.ToString();
			else
				this.WireSource.Step = string.Empty;

			this.RefreshView();
		}

		private void listBoxSourceFlowStepInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxSourceFlowSteps.SelectedItem != null)
				this.WireSource.Step = this.listBoxSourceFlowSteps.SelectedItem.ToString();

			if (this.listBoxSourceFlowStepInterfaces.SelectedItem != null)
				this.WireSource.Pin = this.listBoxSourceFlowStepInterfaces.SelectedItem.ToString();

			this.RefreshView();
		}

		private void listBoxTargetFlowSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (this.listBoxTargetFlowSteps.SelectedItem != null)
			{
				string tKey = this.listBoxTargetFlowSteps.SelectedItem.ToString();
				this.listBoxTargetFlowStepInterfaces.ItemsSource = _EditWireArgument.WireTargets.FlowSteps[tKey];
			}
			else
			{
				this.listBoxTargetFlowStepInterfaces.ItemsSource = null;
				this.listBoxTargetFlowStepInterfaces.Items.Clear();
			}
		}

		private void listBoxTargetFlowSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxTargetFlowSteps.SelectedItem != null)
				this.WireTarget.Step = this.listBoxTargetFlowSteps.SelectedItem.ToString();
			else
				this.WireTarget.Step = string.Empty;

			this.RefreshView();
		}

		private void listBoxTargetFlowStepInterfaces_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxTargetFlowSteps.SelectedItem != null)
				this.WireTarget.Step = this.listBoxTargetFlowSteps.SelectedItem.ToString();

			if (this.listBoxTargetFlowStepInterfaces.SelectedItem != null)
				this.WireTarget.Pin = this.listBoxTargetFlowStepInterfaces.SelectedItem.ToString();

			this.RefreshView();
		}
		#endregion

		#region SingleStepViews Event-Handler
		private void listBoxSourceSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (this.listBoxSourceSingleSteps.SelectedItem != null)
			{
				string tKey = this.listBoxSourceSingleSteps.SelectedItem.ToString();
				this.listBoxSourceSingleStepPins.ItemsSource = _EditWireArgument.WireSources.SingleSteps[tKey];
			}
			else
			{
				this.listBoxTargetFlowStepInterfaces.ItemsSource = null;
				this.listBoxTargetFlowStepInterfaces.Items.Clear();
			}
		}

		private void listBoxSourceSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxSourceSingleSteps.SelectedItem != null)
				this.WireSource.Step = this.listBoxSourceSingleSteps.SelectedItem.ToString();
			else
				this.WireSource.Step = string.Empty;

			this.RefreshView();
		}

		private void listBoxTargetSingleSteps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (this.listBoxTargetSingleSteps.SelectedItem != null)
			{
				string tKey = this.listBoxTargetSingleSteps.SelectedItem.ToString();
				this.listBoxTargetSingleStepPins.ItemsSource = _EditWireArgument.WireTargets.SingleSteps[tKey];
			}
			else
			{
				this.listBoxTargetSingleStepPins.ItemsSource = null;
				this.listBoxTargetSingleStepPins.Items.Clear();
			}
		}

		private void listBoxSourceStepPins_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxSourceSingleSteps.SelectedItem != null)
				this.WireSource.Step = this.listBoxSourceSingleSteps.SelectedItem.ToString();

			if (this.listBoxSourceSingleStepPins.SelectedItem != null)
				this.WireSource.Pin = this.listBoxSourceSingleStepPins.SelectedItem.ToString();

			this.RefreshView();
		}

		private void listBoxTargetSingleSteps_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxTargetSingleSteps.SelectedItem != null)
				this.WireTarget.Step = this.listBoxTargetSingleSteps.SelectedItem.ToString();
			else
				this.WireTarget.Step = string.Empty;

			this.RefreshView();
		}

		private void listBoxTargetSingleStepPins_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (this.listBoxTargetSingleSteps.SelectedItem != null)
				this.WireTarget.Step = this.listBoxTargetSingleSteps.SelectedItem.ToString();

			if (this.listBoxTargetSingleStepPins.SelectedItem != null)
				this.WireTarget.Pin = this.listBoxTargetSingleStepPins.SelectedItem.ToString();

			this.RefreshView();
		}
		#endregion
	}
}
