﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;

namespace AB3.AppDesigner.Helpers
{
	public static class ChartHelper
	{
		/// <summary>Feature 28 (SnapToGrig): Only step = 10 is allowed
		/// </summary>
		/// <param name="value">Value to round to xxx0</param>
		/// <returns>Rounded value to xxx0</returns>
		public static double GetSnapPosition(double value)
		{
			return Math.Round(value / 10, 0) * 10;
		}

		/// <summary>Get the nearest snap point by given point position
		/// </summary>
		/// <param name="position">Point of current position</param>
		/// <returns>Nearest snap point</returns>
		public static Point GetSnapPoint(Point position)
		{
			Point tReturn = new Point();
			tReturn.X = GetSnapPosition(position.X);
			tReturn.Y = GetSnapPosition(position.Y);

			return tReturn;
		}

		public static FDLAttribute GetSizeAttribute(Border border)
		{
			FDLAttribute tSizeAttribute = new FDLAttribute();
			tSizeAttribute.Name = "Size";
			tSizeAttribute.Value = string.Concat(border.Width, ",", border.Height);

			return tSizeAttribute;
		}

		public static FDLAttribute GetPositionAttribute(Border border)
		{
			FDLAttribute tPositionAttribute = new FDLAttribute();
			tPositionAttribute.Name = "Position";
			tPositionAttribute.Value = string.Concat(border.Margin.Left, ",", border.Margin.Top);

			return tPositionAttribute;
		}

		public static FDLAttribute GetSourceEdgeAttribute(string edgeName)
		{
			FDLAttribute tSourceEdgeAttribute = new FDLAttribute();
			tSourceEdgeAttribute.Name = "SourceEdge";
			tSourceEdgeAttribute.Value = edgeName;
			return tSourceEdgeAttribute;
		}

		public static FDLAttribute GetTargetEdgeAttribute(string edgeName)
		{
			FDLAttribute tTargetEdgeAttribute = new FDLAttribute();
			tTargetEdgeAttribute.Name = "TargetEdge";
			tTargetEdgeAttribute.Value = edgeName;
			return tTargetEdgeAttribute;
		}

		public static FDLAttribute GetPointAttribute(string pointName, Point pointPosition)
		{
			FDLAttribute tPointAttribute = new FDLAttribute();
			tPointAttribute.Name = pointName;
			tPointAttribute.Value = string.Concat(pointPosition.X, ",", pointPosition.Y);
			return tPointAttribute;
		}

		public static FDLAttribute GetPointAttribute(string pointName, double x, double y)
		{
			FDLAttribute tPointAttribute = new FDLAttribute();
			tPointAttribute.Name = pointName;
			tPointAttribute.Value = string.Concat(x, ",", y);
			return tPointAttribute;
		}

		public static Border GetSourceBorder(FDLWireDefinition wire)
		{
			if ((wire.Source.Step == "In") && (ChartDefinitionCache.Value.InterfaceDecorators.ContainsKey(wire.Source.Pin)))
				return ChartDefinitionCache.Value.InterfaceDecorators[wire.Source.Pin].View;
			else if (ChartDefinitionCache.Value.StepDecorators.ContainsKey(wire.Source.Step))
				return ChartDefinitionCache.Value.StepDecorators[wire.Source.Step].View;
			else
				return null;
		}

		public static Border GetTargetBorder(FDLWireDefinition wire)
		{
			if ((wire.Target.Step == "Out") && (ChartDefinitionCache.Value.InterfaceDecorators.ContainsKey(wire.Target.Pin)))
				return ChartDefinitionCache.Value.InterfaceDecorators[wire.Target.Pin].View;
			else if (ChartDefinitionCache.Value.StepDecorators.ContainsKey(wire.Target.Step))
				return ChartDefinitionCache.Value.StepDecorators[wire.Target.Step].View;
			else
				return null;
		}

		public static PathGeometry GetArrowPathGeometry(Line line)
		{
			const int tIndicatorTriangleWidth = 8;
			PolyLineSegment tArrowSegment = new PolyLineSegment();
			tArrowSegment.Points.Add(new Point(line.X2, line.Y2));
			tArrowSegment.Points.Add(new Point(line.X2 - tIndicatorTriangleWidth, line.Y2 - tIndicatorTriangleWidth));
			tArrowSegment.Points.Add(new Point(line.X2 - tIndicatorTriangleWidth, line.Y2 + tIndicatorTriangleWidth));
			tArrowSegment.Points.Add(new System.Windows.Point(line.X2, line.Y2));

			PathFigure tArrowPathFigure = new PathFigure();
			tArrowPathFigure.Segments.Add(tArrowSegment);
			PathGeometry tArrowPathGeometry = new PathGeometry();
			tArrowPathGeometry.Figures.Add(tArrowPathFigure);

			//See: http://msdn.microsoft.com/de-de/library/system.math.atan.aspx
			double dy = line.Y2 - line.Y1;
			double dx = line.X2 - line.X1;
			double radians = Math.Atan2(dy, dx);
			double tAngle = radians * (180 / Math.PI);

			if (tAngle != 0)
				tArrowPathGeometry.Transform = new RotateTransform(tAngle, line.X2, line.Y2);

			return tArrowPathGeometry;
		}

		/// <summary>Returns the nearest possible edge of Border based component
		/// </summary>
		/// <param name="border">Border based component (Interface, Remark, SingleStep or FlowStep)</param>
		/// <param name="position">The current position of mouse pointer</param>
		/// <returns>Instance of EdgePosition or null if nothing possible was found</returns>
		public static EdgePosition GetEdgePointByPosition(Border border, Point position)
		{
			double tXSnapFactor = border.Width / 4;
			double tYSnapFactor = border.Height / 4;

			EdgePosition tEdgePosition = null;

			if ((Math.Abs(border.Margin.Left - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top - position.Y) < tYSnapFactor))
				tEdgePosition = new EdgePosition("TopLeft", border.Margin.Left, border.Margin.Top);
			else if ((Math.Abs(border.Margin.Left + border.Width / 2 - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top - position.Y) < tYSnapFactor))
				tEdgePosition = new EdgePosition("Top", border.Margin.Left + border.Width / 2, border.Margin.Top);
			else if ((Math.Abs(border.Margin.Left + border.Width - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top - position.Y) < tYSnapFactor))
				tEdgePosition = new EdgePosition("TopRight", border.Margin.Left + border.Width, border.Margin.Top);
			else if ((Math.Abs(border.Margin.Left + border.Width - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top + border.Height / 2 - position.Y) < tYSnapFactor)) // Right
				tEdgePosition = new EdgePosition("Right", border.Margin.Left + border.Width, border.Margin.Top + border.Height / 2);
			else if ((Math.Abs(border.Margin.Left + border.Width - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top + border.Height - position.Y) < tYSnapFactor)) // BottomRight
				tEdgePosition = new EdgePosition("BottomRight", border.Margin.Left + border.Width, border.Margin.Top + border.Height);
			else if ((Math.Abs(border.Margin.Left + border.Width / 2 - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top + border.Height - position.Y) < tYSnapFactor)) // Bottom
				tEdgePosition = new EdgePosition("Bottom", border.Margin.Left + border.Width / 2, border.Margin.Top + border.Height);
			else if ((Math.Abs(border.Margin.Left - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top + border.Height - position.Y) < tYSnapFactor)) // BottomLeft
				tEdgePosition = new EdgePosition("BottomLeft", border.Margin.Left, border.Margin.Top + border.Height);
			else if ((Math.Abs(border.Margin.Left - position.X) < tXSnapFactor) && (Math.Abs(border.Margin.Top + border.Height / 2 - position.Y) < tYSnapFactor)) // Left
				tEdgePosition = new EdgePosition("Left", border.Margin.Left, border.Margin.Top + border.Height / 2);

			return tEdgePosition;
		}
	}
}
