﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AB3.AppDesigner.DataTypes;
using AB3.DataTypes;

namespace AB3.AppDesigner.Logic
{
	public class EditWireDefinitionArgumentPreparer
	{
		public void In_Input(FDLWireDefinition wireDefinition)
		{
			EditWireArgument tArgument = new EditWireArgument();

			PrepareInterfaces(tArgument);
			PrepareFlowSteps(tArgument);
			PrepareSingleSteps(tArgument);

			Out_Output(wireDefinition, tArgument);
		}

		private void PrepareInterfaces(EditWireArgument editWireArgument)
		{
			foreach (FDLInterfaceDefinition tInterface in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Interfaces.Collection.Values)
			{
				if (tInterface.Direction == Directions.In)
					editWireArgument.WireSources.Interfaces.Add(tInterface.Name);
				else if (tInterface.Direction == Directions.Out)
					editWireArgument.WireTargets.Interfaces.Add(tInterface.Name);
			}

			editWireArgument.WireSources.Interfaces.Sort();
			editWireArgument.WireTargets.Interfaces.Sort();
		}

		private void PrepareFlowSteps(EditWireArgument editWireArgument)
		{
			foreach (FDLStepDefinition tStepDefinition in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Steps.Collection.Values)
			{
				if (tStepDefinition.IsSubFlowStep)
				{
					editWireArgument.WireSources.FlowSteps.Add(tStepDefinition.CustomName, 
						new List<string>(GetInterfacesByFlowDefinitionName(tStepDefinition.TypeName, Directions.Out)));
					editWireArgument.WireTargets.FlowSteps.Add(tStepDefinition.CustomName,
						new List<string>(GetInterfacesByFlowDefinitionName(tStepDefinition.TypeName, Directions.In)));
				}
			}
		}

		private void PrepareSingleSteps(EditWireArgument editWireArgument)
		{
			foreach (FDLStepDefinition tStepDefinition in AppDefinitionCache.Value.FlowDefinitions[FlowDefinitionNameCache.Value].Steps.Collection.Values)
			{
				if (!tStepDefinition.IsSubFlowStep)
				{
					editWireArgument.WireSources.SingleSteps.Add(tStepDefinition.CustomName,
						new List<string>(GetPinsBySingleStepTypeName(tStepDefinition.TypeName, Directions.Out)));
					editWireArgument.WireTargets.SingleSteps.Add(tStepDefinition.CustomName,
						new List<string>(GetPinsBySingleStepTypeName(tStepDefinition.TypeName, Directions.In)));
				}
			}
		}

		private string[] GetInterfacesByFlowDefinitionName(string typeName, Directions interfaceDirection)
		{
			List<string> tReturn = new List<string>();

			if (AppDefinitionCache.Value.FlowDefinitions.ContainsKey(typeName))
			{
				foreach (FDLInterfaceDefinition tInterfaceDefinition in AppDefinitionCache.Value.FlowDefinitions[typeName].Interfaces.Collection.Values)
				{
					if (tInterfaceDefinition.Direction == interfaceDirection)
						tReturn.Add(tInterfaceDefinition.Name);
				}
			}

			tReturn.Sort();
			return tReturn.ToArray();
		}

		private string[] GetPinsBySingleStepTypeName(string name, Directions pinDirection)
		{
			List<string> tReturn = new List<string>();

			foreach (FDLFlowDefinition tFlowDefinition in AppDefinitionCache.Value.FlowDefinitions.Values)
			{
				foreach (FDLStepDefinition tStepDefinition in tFlowDefinition.Steps.Collection.Values)
				{
					if ((!tStepDefinition.IsSubFlowStep) && (tStepDefinition.TypeName == name))
					{
						if (pinDirection == Directions.Out)
						{
							foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
							{
								if (tWireDefinition.Source.Step == tStepDefinition.CustomName)
									tReturn.Add(tWireDefinition.Source.Pin);
							}
						}
						else
						{
							foreach (FDLWireDefinition tWireDefinition in tFlowDefinition.Wires.Collection)
							{
								if (tWireDefinition.Target.Step == tStepDefinition.CustomName)
									tReturn.Add(tWireDefinition.Target.Pin);
							}
						}
					}
				}
			}

			tReturn = tReturn.Distinct().ToList();
			tReturn.Sort();

			return tReturn.ToArray();
		}

		public event Action<FDLWireDefinition, EditWireArgument> Out_Output;
	}
}
