﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;
using AB3.AppDesigner.Adorners;
using AB3.AppDesigner.Helpers;
using AB3.Interfaces;

namespace AB3.AppDesigner.Presenters
{
	public class PartDragDropEventHandler: IConfigure
	{
		private Point _StartPoint;
		private bool _IsDragging = false;
		private StepDragAdorner _DragAdorner = null;

		public void Configure(object args)
		{
			MainWindow.Instance.listViewParts.PreviewMouseLeftButtonDown += listViewParts_PreviewMouseLeftButtonDown;
			MainWindow.Instance.listViewParts.PreviewMouseMove += listViewParts_PreviewMouseMove;
			MainWindow.Instance.AllowDrop = true;
			MainWindow.Instance.gridChart.IsHitTestVisible = true;
			MainWindow.Instance.gridChart.AllowDrop = true;
			MainWindow.Instance.gridChart.PreviewDrop += gridChart_PreviewDrop;
			MainWindow.Instance.Drop += MainWindow_Instance_Drop;
		}

		private void listViewParts_PreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			_StartPoint = e.GetPosition(null);
		}

		private void listViewParts_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if ((e.LeftButton == System.Windows.Input.MouseButtonState.Pressed) && (!_IsDragging))
			{
				Point tCurrentPosition = e.GetPosition(null);

				if (Math.Abs(tCurrentPosition.X - this._StartPoint.X) > SystemParameters.MinimumHorizontalDragDistance ||
					Math.Abs(tCurrentPosition.Y - this._StartPoint.Y) > SystemParameters.MinimumVerticalDragDistance)
				{
					if (MainWindow.Instance.listViewParts.SelectedItems.Count != 0)
					{
						UIElement tElement = (UIElement)e.MouseDevice.DirectlyOver;
						Type tMouseOverType = tElement.GetType();
						if ((tMouseOverType == typeof(TextBlock)) || (tMouseOverType == typeof(Rectangle)) || (tMouseOverType == typeof(Border)))
						{
							string tSelectedItem = MainWindow.Instance.listViewParts.SelectedItem.ToString();
							string tStepType;
							if (tSelectedItem.StartsWith("Step:"))
								tStepType = "SingleStep";
							else
								tStepType = "FlowStep";
							string tCaption = tSelectedItem.Replace("Step: ", string.Empty).Replace("Flow: ", string.Empty);

							this._IsDragging = true;
							if (this._DragAdorner != null)
							{
								this._DragAdorner.Detach();
								this._DragAdorner = null;
							}

							Grid tTopGrid = VisualTreeHelperEx.GetVisualChild<Grid>(MainWindow.Instance);
							this._DragAdorner = new StepDragAdorner(tTopGrid, tStepType, tCaption);
							DataObject tDataObject = new DataObject(tStepType, tCaption);
							DragDropEffects tDragDropEffects = DragDrop.DoDragDrop(MainWindow.Instance.listViewParts, tDataObject, DragDropEffects.Move);
							this._IsDragging = false;
						}
					}
				}
			}
		}

		private void MainWindow_Instance_Drop(object sender, DragEventArgs e)
		{
			if (this._DragAdorner != null)
			{
				this._DragAdorner.Detach();
				this._DragAdorner = null;
			}				
		}

		private void gridChart_PreviewDrop(object sender, DragEventArgs e)
		{
			if (this._DragAdorner != null)
			{
				this._DragAdorner.Detach();
				this._DragAdorner = null;

				string[] tDataFormats = e.Data.GetFormats();
				string tStepType = tDataFormats[0];
				string tTypeName = e.Data.GetData(tStepType).ToString();
				Point tPosition = e.GetPosition(MainWindow.Instance.gridChart);

				Out_Output(tStepType, tTypeName, tPosition.X, tPosition.Y);
			}
		}

		/// <summary>StepType (SingleStep | FlowStep), TypeName, X position, Y Position
		/// </summary>
		public event Action<string, string, double, double> Out_Output;
	}
}
